extensions [ time  profiler pathdir ]

globals
[
  ERROR-CONDITION?
  startdate
  stopdate
  days                ;; = ticks / 24, in order that this need not be calculated several times per day
  seconds-per-day     ;; measures the speed of calculation, allows for stop when executions becomes too slow
  start-time          ;; keeps the information when the model started
  run-length          ;; the time a run used until completion in seconds
  day-before-stopdate ;; to make sure that the simulation stops also in cases when no infected people remain
  weekend?            ;; tells whether the current day is a Saturday or a Sunday
  past-events-counter ;; only for debugging purposes
  daily-moves         ;; to calculate the frequency of movements
  mean-daily-moves    ;; daily mean of the frequency of movements
  NULL                ;; a name for the empty string
;; these globals whose names end in -name are for outputting result files
  logfile-name
  resultfile-name
  profile-name
  worldfile-name
  directory-name
;;
  period             ;; contains information whether there is a lockdown or not
  results            ;; certain results subarea-wise
  log-level          ;; decides what is to be written to ./log.txt
  state-names        ;; for the names of the states
  colour-names       ;; for the colours of the subareas
  day-names          ;; for the startdate and the stopdate monitor and for logging information
  person-zero        ;; the first infectious person
  lockdown-end-at    ;; the incidence rate (new infections during past seven days per 100.000) when lockdown or quarantine end
;; these globals whose names begin with newly- are for calculating moving averages and for calculating the
;; the basic reproduction number estimates
  newly-deceased
  newly-deceased-7-days
  newly-deceased-7-days-ma
  newly-deceased-7-days-ma-max
  newly-deceased-reported
  newly-infected
  newly-infected-reported
  newly-infected-7-days
  newly-infected-7-days-ma
  newly-infected-7-days-ma-max
  newly-infected-4-days
  newly-infected-4-days-ma
  newly-infected-7-days-ma-5-days
  newly-infected-4-days-ma-5-days
  incidence-rate                   ;; the seven-days incidence rate ( = sum newly-deceased-7-days-ma )
;; added 2020-12-10 to keep information of state transitions
  transition-matrix
;; added 2020-10-24 to avoid calculating these values within the R0 plot
  R0-max-ever
  reproduction-number            ;; basic reproduction number = mean of the numbers all current infectious agents have infected
  R04 R07                        ;; estimates of the basic reproduction number
  var-infected                   ;; variance of the numbers all current infectious agents have infected
  dispersion-parameter           ;; parameter of overdispersion
;; added 2010-10-25 to avoid calculating this agentset over and over again
  infectious-people
;;
  total-infectiosity-rate        ;; the current rate of infectious persons among all persons
  pandemic-grows?                ;; signals whether the pandemic is growing
  all-ever-infected              ;; the number of all persons which were ever infected
;; the following globals whose names begin with max- are only used for rescaling the histogram plots
  max-freq-ccs       ;; the frequency of the mode of the distribution of the product of conviviality and carelessness of susceptible agents
  max-freq-cci       ;; the frequency of the mode of the distribution of the product of conviviality and carelessness of infected agents
  max-freq-ccp       ;; the frequency of the mode of the distribution of the product of conviviality, carelessness and individually perceived risk of all agents
  max-freq-pir       ;; the frequency of the mode of the distribution of the individually perceived risk of all agents
  max-freq-ns        ;; the maximum of the frequencies of the distributions of the norm saliences
  max-freq-wii       ;; the maximum of the frequencies of the length of the list whom-I-infected
  max-freq-vis       ;; the maximum of the frequencies of the virus-infectiosity among all but the susceptible people
  max-freq-vii       ;; the maximum of the frequencies of the virus-infectiosity among infectious people
;;
  distances                           ;; a list of all agents' distances to their nearest neighbours (only used at the end of setup)
  mean-distance-to-nearest-neighbour  ;; the mean of the distance to each agent's nearest neighbour
  sdev-distance-to-nearest-neighbour  ;; the standard deviation of the distance to each agent's nearest neighbour
;; changed 2020-12-02 to allow for a two-month infectionless period before the model stops
  end-of-wave?               ;; is true whenever no infectious agent is left
  end-of-run?                ;; is true when the last infectious agent was removed three months ago, when this is true all outstanding events such that the model can stop
  user-halt?                 ;; is true when a user interrupted the model via the GUI and halted it with the option to report all output
  multi-run?                 ;; is true when the model is running headless to avoid some huge output
  no-more-infectious-at      ;; the time when the last infectious agent is removed
  setup-randomseed           ;; the random seed used in the current run for setup and settlement structure, to be able to re-run the current parameter combination
  go-randomseed              ;; the random seed used in the current run after the setup is complete, to be able to run the model with identical settlement structure but different pandemic course
  random-positive-test-rate  ;; the positive-test rate for a random sample of five per cent of the population
  suspect-positive-test-rate ;; the positive-test rate for a non-random sample of all neighbours of all infectious agents within a distance that the sample size if five percent
  max-positive-test-rate     ;; the maximum of either test rate over time (to rescale the respective plot)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2021-08-04 to enable vaccination
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  doses-available
  vaccination-start
  vaccinated
  vaccination-refused
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  all-norms      ;; the names of the norms
  all-norm-types ;; as norms can be grouped to the types afraid of losing health and afraid of losing freedom, these are the types of the norms
  all-actions    ;; the names of the possible actions
;; the globals whose names begin with W are the coefficients of the formula yielding the values of the norm saliences
  WC
  WO
  WNPV
  WP
  WS
  WE
;;
  IDW              ;; the weight of the individual drive as opposed to the normative drive NDW; IDW = 1 - IDW (NDW is set via the user interface)
  WITNESSES        ;; a few agents whose actions are sent to log.txt
  LISTENERS        ;; the number of neighbours which receive norm invocations
  DISCOUNT         ;; a factor by which norm invocations in the agents' memories are discounted
  invocation-count ;; a list for bookkeeping invocations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; output metrics
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  never-met  ;; number of agents which had no contact to infectious agents
  escaped    ;; number of agents which had contact to infectious agents but were not infected
  infected   ;; number of agents infected during the simulation run
  injected   ;; number of agents which entered the simulation as infected (person-zero or returned from abroad
  are-susceptible
  are-infected
  are-ill
  are-recovered
  are-immune
  are-dead
  are-infectious
  median-ccs
  median-ccns
  wear-a-mask
  stay-at-home
  when-max-infected
  n-of-max-infected
  when-max-deceased
  n-of-max-deceased
]

breed [ people human ]
breed [ subareas subarea ]
breed [ squares square ]

people-own
[
  my-subarea                         ;; the subarea the agent usually lives in
  my-home                            ;; the coordinates the agents starts in and returns to every weekend
  conviviality                       ;; a constant feature determining the risk the agent takes
  carelessness                       ;; another such feature
  perceived-risk                     ;; the actually perceived risk, derived either from the percentage of agents in the neighbourhood or of the whole territory
  locally-perceived-risk             ;; the actually perceived risk, derived either from the percentage of agents in the neighbourhood
  my-moves                           ;; counts all moves of agents
;; next two lines added 2021-01-23 to trace the infenction chain staring from an agent
  imported-infectiosity              ;; the virus infectiosity the agent had when it entered the worls (as zero person or returning from abroad )
  my-victims                         ;; a list of all agents whose infection stemmed from this agent
  state                              ;; one of susceptible infected ill recovered dead
  state-history                      ;; a complex list keeping information when and where the state changed
  met-infectious-not-infected        ;; counts how often the agent came near an infectious agent without being infected
  first-move                         ;; time, patch and state of the first move
  prospective-move                   ;; time, patch and state of the next scheduled move
  last-move                          ;; time, patch and state of the last move
;; next line added to implement the case that people return from holidays and are not necessarily infectious
  holidays                           ;; planned day of return from holidays abroad (0: no holidays abroad)
  infectious?                        ;; an additional state which can be combined with some of the states mentioned above
;; next three lines added to implement virus evolution
  my-potential-infector              ;; the agent near me with the highest infectiosity (if there is no virus evolution, just a random agent within distance-for-infection)
  virus-infectiosity                 ;; current infectiosity (infections per contact with this person)
  virus-asymptomaticity              ;; current rate of infected persons becoming asymptomatic after being infected by this person
  positively-tested?                 ;; another additional state mainly used to avoid re-testing
  whom-I-infected                    ;; an agentset keeping all agents infected by this agent
  infection-chain                    ;; a list containing the information about all agents whose infectiosity led to this agent's infection
  future-moves                       ;; a list of scheduled move events, used to avoid future moves before the next move has happened
  immunity-loss-scheduled?           ;; to avoid double scheduling
  witness?                           ;; whether this agent writes its actions to ./log.txt
  my-risk                            ;; the agent's "objective" risk
  my-neighbours                      ;; the agent's current neighbours
  distance-to-my-potential-infector  ;; used to determine which agent will be the one which infects this agent
;; added 2021-08-04 to enable vaccination
  vaccinated?
  vaccination-refused?
  vaccination-day
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  my-norms                        ;; just a copy of all-norms (in fact not necessary in this context, as there is only one breed of agents using norms for decision making)
  my-norm-saliences               ;; a list keeping the agent's current values of the salience of a norm
  my-compliances                  ;; a list keeping the number how often this agent complied with each of the norms
  my-violations                   ;; a list keeping the number how often this agents violated each of the norms
  other-compliances               ;; a list keeping the number how often other agents in the neighbourhood complied with each of the norms
  other-violations                ;; a list keeping the number how often other agents in the neighbourhood violated each of the norms
  punishments                     ;; a list keeping the number how often other agents in the neighbourhood were punished because of violating each of the norms
  sanctions                       ;; a list keeping the number how often other agents in the neighbourhood were sanctioned because of violating each of the norms
  explicit-compliance-invocations ;; a list keeping the number how often this agent received an explicit invocation for each of the norms (as a praise)
  explicit-violations             ;; a list keeping the number how often this agent received an explicit invocation for each of the norms (as a reprimand
  my-party-propensity             ;; this agent's current propensity to go to a party (or elsewhere when not really necessary)
  my-mask-propensity              ;; this agent's current propensity to wear a mask
  my-unmask-propensity            ;; this agent's current propensity to strip its mask off
  my-stay-propensity              ;; this agent's current propensity to stay at home instead of going elsewhere when not really necessary
;;  individual-drive                ;; the drive to take an action as the result of considering just objective risks, 2020-12-17 replaced with local variable
;;  normative-drive                 ;; the drive to take an action as the result of normative consideration, 2020-12-17 replaced with local variable
  last-message-id                 ;; refers to the last message about a punishment, a sanction, or an explicit invocation to make sure that a message updates the normative board only once
  wears-a-mask?                   ;; shorthand for use in the risk-infection procedure
  stays-at-home?                  ;; shorthand for use in the risk-infection procedure
]

subareas-own
[
  area                     ;; the number of patches belonging to this subarea
  population               ;; the number of people whose "home" if in this subarea
  density                  ;; population / area
  subarea-number           ;; to identify a subarea
  colour                   ;; the colour used to make subareas distinguishable in the view
  my-patches               ;; set of all patches belonging to this subarea
  my-people                ;; set of all people originating in this subarea
  my-newly-infected        ;; counts the newly infected agents during the current day
  my-newly-infected-7-days ;; list of the newly infected agents during the past seven days
  my-incidence-rate        ;; number of newly infected agents per 199,999 during the past seven days
  my-newly-deceased        ;; counts the deceased agents of the current day
]

patches-own
[
  my-center   ;; used to identify the subarea to which this patch belongs (necessary to identify patches at the boundary to other subareas)
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; setup procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  set start-time rectify-date
  clear-all
  reset-timer
  set start-time rectify-date
;; initialising global variables
  set end-of-run? false
  set end-of-wave? false
  set user-halt? false
  set NULL ""
  set log-level 0
  set ERROR-CONDITION? false
  set interruptible? false
  set period "normal time"
  set colour-names ( list "red" "orange" "brown" "yellow" "green" "lime" "turquoise" "cyan" "sky" "blue" "violet" "magenta" "pink" )
  set day-names ( list "Mon" "Tue" "Wen" "Thu" "Fri" "Sat" "Sun" )
  set state-names ( list "susceptible" "infected" "ill" "recovered" "immune" "dead" )
  setup-normative-framework
  let run-identifier format 4 behaviorspace-run-number
  set multi-run? behaviorspace-run-number > 0
  set transition-matrix ( list ( list 0 0 0 0 0 0 ) ( list 0 0 0 0 0 0 ) ( list 0 0 0 0 0 0 ) ( list 0 0 0 0 0 0 ) ( list 0 0 0 0 0 0 ) ( list 0 0 0 0 0 0 ) )
;; setting names for output directories and output files
  set directory-name ( word "run_" rectify-date "_" run-identifier )
  pathdir:create directory-name
  set resultfile-name ( word directory-name "/result.csv" )
  set logfile-name ( word directory-name "/log.txt" )
  set profile-name ( word directory-name "/profile.txt" )
  set worldfile-name ( word directory-name "/world.csv" )
;; initialising global variables used for reporting, calculating moving averages and  plotting
  set infectious-people turtle-set nobody
  set newly-deceased-reported 0
  set newly-deceased 0
  set newly-deceased-7-days []
  set newly-infected-reported 0
  set newly-infected 0
  set newly-infected-7-days []
  set newly-infected-4-days []
  set newly-infected-7-days-ma-5-days []
  set newly-infected-4-days-ma-5-days []
  set newly-infected-7-days-ma-max ( list 0 0 )
  set newly-deceased-7-days-ma-max ( list 0 0 )
  set no-more-infectious-at 0
;; these are for initialising the y-range of several histograms
  set max-freq-ccs 0.1
  set max-freq-cci  0.1
  set max-freq-ccp 0.1
  set max-freq-pir 0.1
  set max-freq-ns 0.1
  set max-freq-wii 0.1
  set max-freq-vis 0.1
  set max-freq-vii 0.1
  set R0-max-ever 1.0
;; just for reporting how may events have already occurred
  set past-events-counter 0
  set distances [ 0 ]
;; to have an incidence value for the end of lockdown
  set lockdown-end-at lockdown-at * lockdown-end-at% / 100
;;
  reset-ticks
  file-open logfile-name
  util-flog-observer -2 ( word directory-name ": logging starts, real time is " date-and-time " " rectify-date )
;; setting and keeping the seed used for this run
  set setup-randomseed new-seed
  if constant-seed != "all-random" [ set setup-randomseed setup-seed ]
  random-seed setup-randomseed
  util-flog-observer 0 ( word "setup random seed of this run " run-identifier " was " setup-randomseed )

;; preparing for the start of the simulation
;; this is necessary as the startdate must be fixed before the people can successfully initialise their state-history
;;  reset-ticks
  time:clear-schedule
  set startdate time:create "2020/01/01 00:00"
  if max-months < 1 [ set max-months 1 ]
  set stopdate time:plus startdate max-months "months"
  set day-before-stopdate time:plus stopdate -25 "hours"
  time:anchor-schedule startdate 1 "hour"
  let day-of-week time:get "dayofweek" T TNOW - 1
  set weekend? day-of-week > 4
;; initialising subareas and making subareas initialise patches and people
  util-flog-observer -2  ( word "creating subareas at " timer )
  create-subareas initial-subareas [ subareas-setup ]
  setup-patches
  ask subareas ;; subareas colour their patches and count them to find out their area
  [
    set my-patches patches with [ shade-of? pcolor [ color ] of myself ]
    set area count my-patches
    set my-newly-infected 0
    set my-newly-deceased 0
    set my-newly-infected-7-days []
  ]
  let area-density-population sort-by [ [ m n ] -> item 0 m < item 0 n ] [ ( list area density population ) ] of subareas
  let min-area first first area-density-population
  let max-area first last area-density-population
;; subareas determine their population density according to density-mode and make a preliminary calculation of their population size
  ask subareas
  [
    let this-subarea self
    let this-color color + 3
    set population round ( total-population / initial-subareas )
  ]
;; due to rounding the sum of the population is not equal total-population, check this
  let diff-pop total-population - sum [ population ] of subareas
  ifelse diff-pop < 0
  [ while [ diff-pop < 0 ] [ ask one-of subareas [ set population population - 1 ] set diff-pop diff-pop + 1 ] ]
  [ while [ diff-pop > 0 ] [ ask one-of subareas [ set population population + 1 ] set diff-pop diff-pop - 1 ] ]
;; now the number of the total population is correct, people can be spread over the subarea territories
  util-flog-observer -2 ( word "subareas generated at " timer )
  ask subareas
  [
    let this-subarea self
    let this-color color + 3
    let people-to-sprout round population
    let centers ( patch-set patch-here n-of 19 my-patches )
    ask centers
    [
      sprout-squares 1 [ set shape "circle 3" set size 0.9 set color this-color - 3 ]
      sprout-people ( people-to-sprout / 20 ) [ set my-subarea this-subarea setup-people ]
    ]
    set my-people people with [ my-subarea = myself ]
    let this-population count my-people
    let these-patches my-patches
    if this-population < people-to-sprout [ ask patch-here [ sprout-people ( people-to-sprout - this-population )  [ set my-subarea this-subarea setup-people ] ] ]
    set my-people people with [ my-subarea = myself ]
    ask my-people
    [
      right random 360 forward ( random-beta 2 5 ) * 19.0
      if not member? patch-here these-patches [ move-to one-of these-patches right random 360 forward random-float 0.4 ]
      set my-home ( list xcor ycor )
    ]
    set population count my-people
    set density population / area
  ]
  set area-density-population sort-by [ [ m n ] -> item 0 m < item 0 n ] [ ( list area density population ) ] of subareas
;; calculating the final population density for adjusting infection distances to the size of the population
  util-flog-observer -2 ( word "calculating distance-to-nearest neighbour " timer )
  distance-to-nearest-neighbour
  util-flog-observer -2 ( word "done " timer )
;; initialising plots
  setup-plots
;; placing and initialising person-zero
  let densest-subarea max-one-of subareas [ density ]
  let sparsest-subarea min-one-of subareas [ density ]
  if zero-person = "densest" [ set person-zero one-of people with [ my-subarea = densest-subarea ] ]
  if zero-person = "sparsest" [ set person-zero one-of people with [ my-subarea = sparsest-subarea ] ]
  if zero-person = "random" [ set person-zero one-of people ]
  if zero-person = "center" [ set person-zero min-one-of people [ xcor * xcor + ycor * ycor ] ]
  set end-of-run? false
  set end-of-wave? false
  set witnesses n-of 2 people
  ask witnesses [ set witness? true ]
  ask person-zero
  [
    set state-history lput ( list "zero-person" ) state-history
    set-state "infected"
    set-infectious
    set infection-chain lput ( list "zero-person" T TNOW patch-here size shape color ) infection-chain
    set imported-infectiosity infections-per-contact
    set my-victims []
    set past-events-counter past-events-counter - 1
    set newly-infected newly-infected + 1
    ask my-subarea [ set my-newly-infected my-newly-infected + 1 ]
    set all-ever-infected all-ever-infected + 1
    report-tested
  ]
;; completing the task of initialising people
  set-normative-predispositions
  util-flog-observer -2 ( word "setup complete " timer )
  reset-ticks
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2021-08-04 to enable vaccination
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set vaccination-start 355
  set vaccinated 0
  set vaccination-refused 0
end

;; calculates the mean and standard deviation of the distance of all agents to their nearest neighbpur
to distance-to-nearest-neighbour
  set distances [ distance min-one-of other people [ distance myself ] ] of people
  set mean-distance-to-nearest-neighbour mean distances
  set sdev-distance-to-nearest-neighbour standard-deviation distances
end

;; initialising the global variables for the normative mode of the model
to setup-normative-framework
  set NULL ""
  set all-actions    ( list "PUTTING ON MASK" "GOING TO PARTY" "PUTTING OFF MASK" "STAYING AT HOME" )
  set all-norms      ( list "NORM: WEAR A MASK" "NORM: BREATHE FREELY" "NORM: STAY AT HOME" "NORM: GO TO PARTY" )
  set all-norm-types ( list "FEAR PANDEMIC"     "FEAR OPPRESSION"      "FEAR PANDEMIC"      "FEAR OPPRESSION"   )
;; the weights for calculating the salience of a norm according to ODD, Appendix "Minding Norms in a Nutshell"
  set WC 0.99
  set WO 0.33
  set WNPV 0.66
  set WP 0.33
  set WS 0.99
  set WE 0.99
;; NDW (normative drive weight) is set in the GUI, IDW (individual drive weight) is just 1-NDW
  set IDW 1 - NDW
  set invocation-count ( list ( list "NORM: WEAR A MASK   " ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: BREATHE FREELY" ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: STAY AT HOME  " ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: GO TO PARTY   " ( list 0 0 0 0 0 0 0 0 ) ) )
  set LISTENERS 5
  set DISCOUNT 0.95
end

;; setup procedures for the different agent types follow
to subareas-setup
  set subarea-number who mod initial-subareas
  set label subarea-number
  set color item subarea-number [ 13 23 33 43 53 63 73 83 93 103 113 123 133 ]
  set label-color color - 3
  set colour item subarea-number colour-names
  set shape "circle 3"
  set size 3
  move-to one-of patches with [ not any? other turtles-here ]
end

to setup-patches
  ask patches
  [
    set my-center min-one-of subareas [ distance myself ]
    set pcolor 6 + [ color ] of my-center
  ]
end

to setup-people
  set color white
  set shape "person"
  set size 0.6

;; initialising agents' instance variables
;; in the heterogeneous case the following two variables are beta distributed between 0 and 1 symmetric with respect to 0.5
;; such that mean, mode and median are 0.5
  ifelse heterogeneity = "none" or heterogeneity = "wrt norm" [ set conviviality 0.5 ] [ set conviviality random-beta 2 2 ]
  ifelse heterogeneity = "none" or heterogeneity = "wrt norm" [ set carelessness 0.5 ] [ set carelessness random-beta 2 2 ]
  set future-moves 0
  set immunity-loss-scheduled? false
  set state "susceptible"
  set state-history []
  set first-move []
  set last-move []
  set infectious? false
  set met-infectious-not-infected 0
  set my-home ( list xcor ycor )
  set positively-tested? false
  set infection-chain []
  set whom-I-infected []
  set witness? false
  set wears-a-mask? false
  set stays-at-home? false
  set distance-to-my-potential-infector 100 ;; 100 is the maximum possible distance, minimum is determined later
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2021-01-04 to provide for virus evolution
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set virus-infectiosity infections-per-contact
  set virus-asymptomaticity asymptomatic-per-infected
  set my-potential-infector nobody
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2020-11-17 to provide for the case that people return from holidays abroad and might be infectious, slightly changed 2021-08-17 to have more travel months
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  let holiday-period random 24
  ifelse member? holiday-period [ 3 9 10 15 21 22 ]  [ set holidays ( holiday-period - 1 ) * 30 + random-float 30 ]  [ set holidays 0 ]
;; added 2021-01-23
  set imported-infectiosity 0
  set my-victims []
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2021-08-04 to enable vaccination
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set vaccinated? false
  set vaccination-refused? false
  set vaccination-day 999
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; initialising the normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set my-norms all-norms
  set my-norm-saliences n-values length my-norms [ 0.0 ]
  set my-compliances n-values length my-norms [ 0 ]
  set my-violations n-values length my-norms [ 0 ]
  set my-party-propensity 0
  set my-mask-propensity 0
  set my-unmask-propensity 0
  set my-stay-propensity 0
  set other-compliances n-values length my-norms [ 0 ]
  set other-violations n-values length my-norms [ 0 ]
  set punishments n-values length my-norms [ 0 ]
  set sanctions n-values length my-norms [ 0 ]
  set explicit-compliance-invocations n-values length my-norms [ 0 ]
  set explicit-violations n-values length my-norms [ 0 ]
end

;; filling the agents' memories with norm relevant information from the time before the simulation began
to set-normative-predispositions
;; BACKGROUND is the mean of initial invocations of fear pandemic norms, initial compliance dispositions (LNP) of these norms are
;; beta distributed with mean BACKGROUND and range 20; if LNP turns out to be negativem it is set to 0
;; and the initial violation disposition (TNP) is set to the absolute value of LNP; for fear oppression norms, the opposite applies
  ask people
  [
    let my-norm-compliance-dispositions []
    let my-norm-violation-dispositions []
    foreach my-norms
    [ norm ->
      let individual-LNP 0
      ifelse heterogeneity = "none" or heterogeneity = "wrt risk"
      [ set individual-LNP BACKGROUND ]
      [ set individual-LNP round ( BACKGROUND + ( random-beta 2 2 - 0.5 ) * 20 ) ]
      let individual-TNP 0
      if individual-LNP < 0
      [
        set individual-TNP 0 - individual-LNP
        set individual-LNP 0
      ]
      util-flog 6 ( word "Norm: " norm ", BACKGROUND: " BACKGROUND ", individual-LNP: " individual-LNP ", individual-TNP: " individual-TNP )
      let np position norm all-norms
      if item np all-norm-types = "FEAR PANDEMIC"
      [
        set my-norm-compliance-dispositions lput individual-LNP my-norm-compliance-dispositions
        set my-norm-violation-dispositions lput individual-TNP my-norm-violation-dispositions
      ]
      if item np all-norm-types = "FEAR OPPRESSION"
      [
        set my-norm-compliance-dispositions lput individual-TNP my-norm-compliance-dispositions
        set my-norm-violation-dispositions lput individual-LNP my-norm-violation-dispositions
      ]
    ]
    set my-compliances my-norm-compliance-dispositions
    set my-violations my-norm-violation-dispositions
    set other-compliances my-norm-compliance-dispositions
    set other-violations my-norm-violation-dispositions
    set punishments  my-norm-violation-dispositions
    set sanctions  my-norm-violation-dispositions
    set explicit-compliance-invocations my-norm-compliance-dispositions
    set explicit-violations  my-norm-violation-dispositions
    if witness? [ util-flog 7 ( word "initial disposition comply: " my-norm-compliance-dispositions " violate: " my-norm-violation-dispositions ) ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; procedures to start the simulation and keep it running
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to go
  set go-randomseed new-seed
  if constant-seed = "all-constant" [ set go-randomseed go-seed ]
  random-seed go-randomseed
  util-flog-observer -2 ( word "go random seed for this run is " go-randomseed  )
  set run-length run-length + timer
  reset-timer
  set interruptible? false
  set end-of-run? false
  set end-of-wave? false
  if not multi-run?  ;; if NetLogo runs in behaviour space the profiler is switched off
  [
    profiler:start
    profiler:reset
    profiler:start
  ]
  prepare-results
  time-schedule-days ( time:plus startdate 2E-5 "minutes" ) ;; the next daily statistic is scheduled
  util-flog-observer -2 ( word directory-name ": model started at " T TNOW " the calendar length is " time:size-of-schedule ", real time is " date-and-time " " rectify-date )
;; next line added 2020-11-06 to allow for superspreading events
  ifelse superspread? [ prepare-a-superspreading-event ] [util-flog-observer -2 ( word "no superspreading event" ) ]
  ask people ;; all agents define their first moving event
  [
    let my-next-move ( time:plus T TNOW random-float 1.0 "days"  )
    time-schedule-event self [ [] -> move-around ] "move around" my-next-move
    set first-move ( list ticks state my-next-move patch-here )
    set last-move ( list ticks state my-next-move patch-here )
  ]
;; for the variant with infections later during the run, these events are scheduled, this part changed 2020-11-17 to adapt for those who return twice
  ifelse infectious-from-abroad != "none" [ prepare-returning-travellers ]   [ util-flog-observer -2 ( word "no infections from abroad" ) ]
  util-flog-observer -2 ( word "people activated for the first time at " T TNOW " the calendar length is " time:size-of-schedule )
;; more events will be scheduled in the course of the simulation, the simulation ends at stopdate, but if interrupted or no infectious agents left
;; scheduled events are deleted, and stopdate is reached
  time:go
  set end-of-run? true
  util-flog-observer -2 ( word "run stopped at " T TNOW " the calendar length is " time:size-of-schedule ", end-of-run? is " end-of-run? ", stopdate is " stopdate "/" day-before-stopdate  )
  report-final-results
end

;; code added 2020-11-17 and moved to this procedure 2020-12-11
to prepare-returning-travellers
  let max-travellers total-population * travellers
  let returning-travellers n-of max-travellers people
  let other-returning-travellers n-of ( 3 * max-travellers ) people
  let more-returning-travellers n-of ( 2 * max-travellers ) people
  ( ifelse
    infectious-from-abroad = "months 2..11"
    [
      ask returning-travellers
      [ time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW ( 31 + random-float 304 ) "days" ) ]
    ]
    infectious-from-abroad = "months 7..8"
    [
      ask returning-travellers
      [ time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW ( 182 + random-float 62 ) "days" ) ]
    ]
    infectious-from-abroad = "months 9..11"
    [
      ask returning-travellers
      [ time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW ( 244 + random-float 91 ) "days" ) ]
    ]
    infectious-from-abroad = "months 2..3 6..9"
    [
      ask returning-travellers
      [
        time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW ( 31 + random-float 60 ) "days" )
      ]
      ask other-returning-travellers
      [
        time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW ( 152 + random-float 124 ) "days" )
      ]
    ]
    infectious-from-abroad = "holidays"
    [
      util-flog-observer -2 ( word "scheduled " count up-to-n-of max-travellers people with [ holidays > 0 ] " to return home infected from holidays" )
      ask up-to-n-of max-travellers people with [ holidays > 0 ]
      [
        time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW holidays "days" )
      ]
    ]
    infectious-from-abroad = "continuous"
    [
      util-flog-observer -2 ( word "scheduled " count up-to-n-of max-travellers people " to return home infected at any time" )
      ask up-to-n-of max-travellers people
      [
        let return-day random-float ( max-months * 30 )
        time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW return-day "days" )
      ]
    ]
    infectious-from-abroad = "seasonal"
    [
      util-flog-observer -2 ( word "scheduled " count up-to-n-of max-travellers people " travellers, " %continuous "  % of them to return home infected at any time, the others to return from holidays" )
      ask up-to-n-of max-travellers people
      [
        let return-day holidays
        if ( random 100 < %continuous ) [ set return-day random-float ( max-months * 30 ) ]
        time-schedule-event self [ [] -> return-from-abroad-infectious ] "return-from-abroad" ( time:plus T TNOW return-day "days" )
      ]
    ]
  )
end


;; next two procedures added 2020-11-05 to implement superspreading events
to prepare-a-superspreading-event
  ask one-of ( max-n-of 50 people [ conviviality * carelessness ] )
  [
    let location patch-here
    let event-time time:plus T TNOW ( 0.5 + random-float 14) "days"
    time:schedule-event self [ [] -> start-a-superspreading-event ] event-time
    util-flog -2 ( word "superspreading event announced for " event-time " at " location )
    if not multi-run? [ output-print ( word "superspreading event planned for " time:show event-time "yyyy-MM-dd HH:mm" " at " [ ( word pxcor "/" pycor ) ] of location ) ]
  ]
  let preparation-time time:plus T TNOW ( 0.5 + random-float 28 ) "days"
  time:schedule-event "observer" [ [] -> prepare-a-superspreading-event ] preparation-time
  util-flog-observer -2 ( word "superspreading event planned at " preparation-time )
end

to start-a-superspreading-event
  let newly-infected-here 0
  let participants max-n-of 50 ( people in-radius 10 ) [ conviviality * carelessness ]
  let this-patch patch-here
  let this-pxcor pxcor let this-pycor pycor
  let n-of-participants count participants
  file-open logfile-name
  file-show ( word "superspreading event begins with " n-of-participants " participants, among them " count participants with [ infectious? ] " infectious" )
  ifelse any? participants with [ infectious? ]
  [
    ask participants
    [
      set witness? true
      setxy ( this-pxcor + random-float-in-range -0.5 0.5 ) ( this-pycor + random-float-in-range -0.5 0.5 )
    ]
    ask participants with [ state = "susceptible" ]
    [
      util-flog 5 ( word "now risks infection" )
      risk-infection
      if state = "infected"
      [
        util-flog 5 ( "is now infected" )
        set newly-infected-here newly-infected-here + 1
      ]
      time-schedule-event self [ [] -> go-home ] "go-home" ( time:plus T TNOW ( 3 + random-float 5 ) "hours" )
      set witness? false
    ]
    set pcolor red
    file-show ( word "event at " this-patch " caused " newly-infected-here " infected out of " n-of-participants )
    if not multi-run? [ output-print ( word "event at " this-patch " caused " newly-infected-here " infected out of " n-of-participants ) ]
  ]
  [
    set pcolor green
    if not multi-run? [ output-print ( word "no infectious here at " this-patch ) ]
    file-show ( word "superspreading event at " this-patch " ended without infections" )
  ]
end


;; a utility which moves an agent to an allowable place depending on the kind of period (lockdown or not) and its own state as far-commuter
to distance-as-regulated
  if period = "normal time"
  [
    ifelse random-float 1.0 < far-commuter
    [ forward long-distance util-flog 50 ( word "moved " long-distance " " ifelse-value infectious? [ "" ] [ "not " ] "infectious." ) ]
    [ forward short-distance ]
  ]
  if period = "lockdown"
  [
    ifelse random-float 1.0 < far-commuter / 2
    [ forward short-distance util-flog 50 ( word "moved " ( short-distance ) " " ifelse-value infectious? [ "" ] [ "not " ] "infectious." ) ]
    [ forward 0.1 ]
  ]
  if period = "quarantine" [ if any? people in-radius 5  with [ infectious? ] [ forward 0 ] ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; agents' procedures to move around and change their infection state
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move-around ;; general procedure to branch to one of the special move procedures
  set future-moves future-moves - 1                    ;; this line to prevent an agent from scheduling more than one move in the future
  set past-events-counter past-events-counter + 1
  if state != "ill" and state != "dead"
  [
     update-normative-board
     move-all-mobile
  ]

  ( ifelse
    state = "susceptible" [ move-susceptible ]
    state = "infected" [ move-infected ]
    state = "ill" [ move-ill ]
    state = "recovered" [ move-recovered ]
    state = "dead" [ move-dead ]
    state = "immune" [ move-immune ]
    [ util-flog -2 ( word state " is not an allowable state" ) set ERROR-CONDITION? true ]
  )
  if infectious? [ set shape "default" ]
  if abs(xcor) > 49 or abs (ycor) > 49 or weekend?
  [ go-home ]
end

to move-all-mobile  ;; common procedure for all which are mobile (not ill and not dead), also prevents agents to move beyond the boundaries of the world
  ifelse abs(xcor) > 49 or abs (ycor) > 49 or weekend?
  [ go-home ]
  [
    right random 360
    distance-as-regulated
;;    set state-history lput ( list state T TNOW patch-here size shape color ) state-history
    if empty? first-move [ set first-move ( list ticks state T TNOW patch-here ) ]
    set last-move ( list ticks state T TNOW patch-here )
  ]
  set daily-moves daily-moves + 1
  set my-moves my-moves + 1
end

to ask-for-vaccination
  if doses-available > 0 and not vaccinated? and not vaccination-refused?
  [
    ifelse carelessness < 0.7
    [
      set vaccinated? true
      set vaccinated vaccinated + 1
      set vaccination-day days
      set doses-available doses-available - 1
    ]
    [
      set vaccination-refused vaccination-refused + 1
      set vaccination-refused? true
    ]
  ]
end

to move-susceptible  ;; procedure for susceptible agents
;; changed 2021-01-04 to introduce virus evolution
  let potential-infectors people in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour ) with [ infectious? ] ;; and random-float 1.0 < infections-per-contact
  util-flog 67 ( word "move-susceptible entered with " count potential-infectors " potential infectors" )
  if vaccination? and days > vaccination-start
  [
    ask-for-vaccination
  ]
  if vaccinated? and days - vaccination-day > 14
  [
    set-state "immune"
  ]
  if any? potential-infectors and state = "susceptible"
  [
    set my-potential-infector max-one-of potential-infectors [ virus-infectiosity ]
    util-flog 67 ( word "risk-infection called with my-potential infector being " my-potential-infector )
    ifelse random-float 1.0 < [ virus-infectiosity ] of my-potential-infector
    [ risk-infection ]
    [
      util-flog 67 "virus-infectiosity too low"
    ]
    util-flog 67 ( word "returned from risk-infection with my-potential infector being " my-potential-infector )
  ]
  ifelse future-moves < 1
  [
    let my-next-move next-move
    util-flog 67 ( word "next move scheduled for " my-next-move )
    time-schedule-event self [ [] -> move-around ] "move around" my-next-move
  ]
  [ util-flog 67 ( word "future-moves = " future-moves ", not scheduled." ) ]
end

to move-infected ;; procedure for infected agents to fall ill or recover asymptomatically
  ifelse random-float 1.0 < asymptomatic-per-infected
  [ time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" ) ]
  [ time-schedule-event self [ [] -> fall-ill ] "fall ill" ( time:plus T TNOW incubation-time "days" ) ]
  if random-float 1.0 < tested-per-infected and not positively-tested?
  [
    set positively-tested? true
    if random-float 1.0 < reporting-rate [ report-tested util-flog 9 ( "report-tested called" ) ]
    go-home
  ]
  if witness? [ util-flog 67 ( word "leaving move-infected" ) ]
end

to move-ill ;; procedure for sick agents to recover or die
  time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" )
  if not positively-tested?
  [
    set positively-tested? true
    if random-float 1.0 < reporting-rate [ report-tested util-flog 9 ( "report-tested called" ) ]
  ]
end

to move-recovered  ;; procedure for recovered agents to continue moving at time=next-move
  let time-till-immunity random-float-in-range time-till-immunity-min time-till-immunity-max
  let immunity-start-at time:plus T TNOW time-till-immunity "days"
  util-flog 29 ( word "will become immune at " ( immunity-start-at ) )
  time-schedule-event self [ [] -> become-immune ] "become immune" ( immunity-start-at )
  let my-next-move next-move
  if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" my-next-move ]
end

to move-dead ;; proedure to reset some instance variable after the end of the agent's life
  set positively-tested? false
  set infectious? false
  set shape "Cross"
  set color black
end

to move-immune  ;; procedure to have immune agents lose their immunity
  let immunity-duration ( time:plus T TNOW ( random-float-in-range immunity-duration-min immunity-duration-max ) "months" )
  let my-next-move next-move
  if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" my-next-move ]
  if not immunity-loss-scheduled? [ time-schedule-event self [ [] -> lose-immunity ] "lose-immunity" immunity-duration ]
  set immunity-loss-scheduled? true
end




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; health related procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to lose-immunity  ;; what happens if an agent loses immunity
  set past-events-counter past-events-counter + 1
  set-state "susceptible"
  if future-moves < 1
  [
    let my-next-move next-move
    time-schedule-event self [ [] -> move-around ] "move around" my-next-move
  ]
end

to-report next-move
  let span 0.5 + ( random-beta 2 5 ) * 2 ;; this means at least half a day, at most 2.5 days, with a mode at about 0.63 and a median at about 0.72 days
  let frequency conviviality * carelessness
  report  ( time:plus T TNOW ( span / ( frequency * move-speed ) ) "days" )
end

to risk-infection  ;; taking the risk of being infected, dependent on own precautions and distance to nearest infectious agent and its infectiosity
  util-flog 67 "risk-infection entered"
;; the agent arrived in the neighbourhood of one or more infectious agents
;; the infection risk depends on how convivial and how careless an agent is
;; ( 1 - conviviality ) is the propensity to avoid crowding
;; ( 1 - carelessness ) is the propensity to take care
;; the product of the two is the probability to escape infection
;; hence one minus this product is the probability not to escape the infection, i.e. to be infected
  let risk-to-be-infected ( 1 - ( 1 - conviviality ) * ( 1 - carelessness ) )
;; and is reduced by wearing a mask or staying at home
  if wears-a-mask? [ set risk-to-be-infected risk-to-be-infected * 0.3]
  if stays-at-home? [ set risk-to-be-infected risk-to-be-infected * 0.1 ]
  ifelse risk-to-be-infected > random-float 1.0 and any? infectious-people
  [ ;; the risk has become manifest
    util-flog 1 ( word "risk " precision risk-to-be-infected 4 " " precision ( conviviality * carelessness ) 4 " infected" )
    set distance-to-my-potential-infector distance my-potential-infector
    util-flog 67 ( word "my potential infector is " my-potential-infector ", " distance-to-my-potential-infector " patches away at " [ ( list xcor ycor ) ] of my-potential-infector )
;; next line changed 2020-11-06 when superspreading events were introduced to make sure that an infector is actually found
    let my-infector my-potential-infector
;; in the next two lines, my-infector copies its infection chain into mine and adds me to the list of those whom it infected
    set infection-chain lput ( list my-infector T TNOW patch-here size shape color ) [ infection-chain ] of my-infector
    let the-infected-agent self
    ask my-infector
    [
      set whom-I-infected lput myself whom-I-infected
      let origin nobody
      ifelse length infection-chain = 1
      [ set origin self ]
      [ set origin first item 1 infection-chain ]
      ask origin [ set my-victims lput the-infected-agent my-victims ]
      util-flog 77 ( word "origin of this infection chain is " origin ", my victims are " my-victims ", chain goes back to " first item 0 infection-chain )
      util-flog 67 ( word " infected " myself ", history is " state-history )
    ]
;;    if [ witness? ] of my-infector
;;    [
      util-flog 67 ( word "my infector is " my-infector " whose history is " [ infection-chain ] of my-infector " and who infected " [ whom-I-infected ] of my-infector )
;;    ]
;; the infected agent inherits its infector's infectiosity and asymptomaticity and changes is according to the mutation rate
    let something random-float-in-range ( 0 - mutation-rate ) mutation-rate
    set virus-infectiosity min ( list 1 ( [ virus-infectiosity ] of my-infector + something ) )
    set virus-asymptomaticity min ( list 1 ( [ virus-asymptomaticity ] of my-infector + something ) )
    set-state "infected"
    set newly-infected newly-infected + 1
    ask my-subarea [ set my-newly-infected my-newly-infected + 1 ]
    set all-ever-infected all-ever-infected + 1
    let time-to-infectiosity ( ifelse-value ;; changed 2021-01-11 to allow for a right-skewed distribution, too
      infection-distribution = "beta 2 5" [ ( time:plus T TNOW ( time-to-infectiosity-min + random-beta 2 5 * ( time-to-infectiosity-max - time-to-infectiosity-min ) ) "days" ) ]
      infection-distribution = "beta 5 2" [ ( time:plus T TNOW ( time-to-infectiosity-min + random-beta 5 2 * ( time-to-infectiosity-max - time-to-infectiosity-min ) ) "days" ) ]
      infection-distribution = "uniform"  [ ( time:plus T TNOW ( random-float-in-range time-to-infectiosity-min time-to-infectiosity-max ) "days" ) ]
    )
    let my-next-move next-move
    util-flog 5 ( word "is now infected and will move again at " my-next-move " and become infectious at " time-to-infectiosity )
    time-schedule-event self [ [] -> set-infectious ] "set-infectious" time-to-infectiosity
    if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" my-next-move ]
    if random-float 1.0 < tested-per-infected and not positively-tested?
    [
      set positively-tested? true
      if random-float 1.0 < reporting-rate [ report-tested util-flog 9 ( "report-tested called" ) ]
    ]
  ]
  [ ;; the risk has not realised
    util-flog 67 ( word "risk " precision risk-to-be-infected 4 " perceived: " precision perceived-risk 4 " individual: " precision ( conviviality * carelessness ) 4 " not infected; "  )
    set state-history lput ( list "not infected" T TNOW patch-here size shape color ) state-history
    set met-infectious-not-infected met-infectious-not-infected + 1
    if future-moves < 1
    [
      let my-next-move next-move
      util-flog 67 ( word "move-around scheduled for " my-next-move )
      time-schedule-event self [ [] -> move-around ] "move around" my-next-move
    ]
  ]
  util-flog 67 "risk-infection left"
end

to set-infectious   ;; collects all measures taken when a person is infectious in the course of its own being infected
  set past-events-counter past-events-counter + 1
  if not ( state = "dead" or state = "immune" or infectious? )
  [
    set infectious? true  set shape "default"
    util-flog 29 ( word "will lose infectiosity at " ( time:plus T TNOW infectiosity-duration "days" ) )
    time-schedule-event self [ [] -> lose-infectiosity ] "lose infectiosity" ( time:plus T TNOW infectiosity-duration "days" ) ;; added 2020-12-12
    set size 2
    ask people in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour ) [ move-around ]
    set state-history lput ( list "infectious" T TNOW patch-here size shape color ) state-history
  ]
end

to lose-infectiosity
  if infectious? and state != "dead"
  [
    set infectious? false
    set shape state-shape
    set size 0.6
    util-flog 29 ( word "lost infectiosity at " T TNOW )
    let my-next-move next-move
    time-schedule-event self [ [] -> move-around ] "move around" my-next-move
  ]
end


to return-from-abroad-infectious  ;; as of 2020-11-19, collects all measures taken when a person is infectious without being infected within the simulation
;; if auto-lockdown = "external", the probability of entering the simulation in the "infected" state is reduced to 0.25,
;; during quarantine it is reduced to 0, agents who were infected before leave this procedure immediately
;; if auto-lockdown = "internal" of "off", there are no special regulations for introducing infected agents as if they returned from abroad
  set past-events-counter past-events-counter + 1
  ifelse auto-lockdown = "external" and ( ( period = "lockdown" and random-float 1.0 > 0.25 ) or period = "quarantine" )
  [
    util-flog -2 ( word "returning from abroad during " period ", quarantined whether infectious or not, no danger for others" )
  ]
  [
    if ( state = "susceptible" )
    [
      set infection-chain lput ( list "from abroad" T TNOW patch-here size shape color ) infection-chain
      set state-history lput ( list "from abroad" ) state-history
      set-state "infected"
      set all-ever-infected all-ever-infected + 1
      set newly-infected newly-infected + 1
      set infectious? true  set shape "default"
      set witness? true
;; next two lines inserted 2021-01-18 to include infections from abroad into the mutation process
      let something random-float-in-range ( 0 - mutation-rate ) ( 2 * mutation-rate )
      ifelse not empty? [ virus-infectiosity ] of other people with [ infectious? ]
      [
        set virus-infectiosity    min ( list 1 ( mean [ virus-infectiosity ]    of other people with [ infectious? ] + something ) )
        set virus-asymptomaticity min ( list 1 ( mean [ virus-asymptomaticity ] of other people with [ infectious? ] + something ) )
      ]
      [
        set virus-infectiosity    min ( list 1 ( infections-per-contact + something ) )
        set virus-asymptomaticity min ( list 1 ( asymptomatic-per-infected + something ) )
      ]
      set imported-infectiosity virus-infectiosity
      set my-victims []
      set size 2
      set state-history lput ( list "infectious from abroad " T TNOW patch-here size shape color ) state-history
      set first-move ( list ticks state T TNOW patch-here )
      let my-next-move next-move
      if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" my-next-move ]
      if not multi-run? [ output-show ( word T TNOW " infected from abroad" ) ]
      util-flog -2 ( word T TNOW " infected from abroad with virus infectiosity " virus-infectiosity )
      let susceptible-neighbours people with [ state = "susceptible" ] in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour )
      util-flog -2 ( word "has the following " count susceptible-neighbours " susceptible agents in its neighbourhood:" )
      util-flog -2 ( word [ who ] of susceptible-neighbours )
    ]
  ]
end

to fall-ill    ;; collects all measures taken when an agent falls ill
  set past-events-counter past-events-counter + 1
  ifelse state = "ill" or state = "dead" or state = "immune" or state = "recovered"
  [
    util-flog 15 ( word "fall-ill cannot be entered by people with " state )
  ]
  [
    let my-hospital one-of [ neighbors ] of my-subarea
    setxy [ pxcor ] of my-hospital [ pycor ] of my-hospital
    set-state "ill"
    time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" )
    if not positively-tested?
    [
      set positively-tested? true
      report-tested util-flog 9 ( "report-tested called" )
    ]
  ]
end

to go-home  ;; return to original patch every weekend, after returning from hospital or when dead
  setxy first my-home last my-home
;;  set state-history lput ( list "went home " T TNOW patch-here size shape color ) state-history
end

to become-immune  ;; collects all changes to be made when an agents becomes immune
  ifelse state = "immune" [ util-flog 67 "is already immune" ]
  [
    set past-events-counter past-events-counter + 1
    util-flog 29 ( word "should become immune now at " T TNOW )
    set-state "immune"
    set infectious? false
    set shape state-shape
    set size 0.6
    set positively-tested? false
    set whom-I-infected lput "now immune" whom-I-infected
  ]
end

to recover-or-die   ;; collects all state changes when an infected agents recovers or dies (is removed, in the SIR terminology)
  set past-events-counter past-events-counter + 1
  ifelse state = "recovered" or state = "dead" or state = "immune"
  [
    util-flog 20 ( word "recover-or-die cannot be entered by people with " state )
  ]
  [
    ifelse random-float 1.0 < case-fatality-rate and state = "ill"
    [
      go-home
      set-state "dead"
      set infectious? false
      set shape "Cross"
      set size 2
      set newly-deceased newly-deceased + 1
      ask my-subarea [ set my-newly-deceased my-newly-deceased + 1 ]
      report-death
    ]
    [
      go-home
      set-state "recovered"
      if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" ( time:plus T TNOW 4 "days" ) ]
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; reporting procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to report-final-results
  file-close-all
  file-open resultfile-name                      ;; outputs subarea related statistics
  foreach n-values initial-subareas [ n -> n ] [ list-item ->        ;; the headline
    file-type "time, subarea,"
    file-type ( word "area" format 2 list-item ",")
    file-type ( word "density" format 2 list-item "," )
    file-type ( word " population" format 2 list-item ", " )
    file-type ( word " susceptible" format 2 list-item ", " )
    file-type ( word " infected" format 2 list-item ", " )
    file-type ( word " ill" format 2 list-item ", " )
    file-type ( word " recovered" format 2 list-item ", " )
    file-type ( word " immune" format 2 list-item ", " )
    file-type ( word " dead" format 2 list-item ", " )
    file-type ( word " infectious" format 2 list-item ", " )
    file-type ( word " newly-infected" format 2 list-item ", " )
    file-type ( word " newly-deceased" format 2 list-item ", " )
    file-type ( word " R0" format 2 list-item ", " )
  ]
  file-print ""
  foreach results                            ;; the final statistics for each subarea
  [ ?1 ->
    let daily-result ?1
    ( foreach n-values initial-subareas [ ??1 -> ??1 ] daily-result
    [ [??1 ??2] ->
      let subarea-result ??2
      file-type ( word item 0 subarea-result"," ) ;; time
      file-type ( word item 1 subarea-result"," ) ;; subarea
      file-type ( word item 4 subarea-result"," ) ;; area
      file-type ( word item 5 subarea-result"," ) ;; density
      file-type ( word item 6 subarea-result"," ) ;; population
      file-type ( word item 7 subarea-result"," ) ;; susceptible
      file-type ( word item 8 subarea-result"," ) ;; infected
      file-type ( word item 9 subarea-result"," ) ;; ill
      file-type ( word item 10 subarea-result"," ) ;; recovered
      file-type ( word item 11 subarea-result"," ) ;; immune
      file-type ( word item 12 subarea-result"," ) ;; dead
      file-type ( word item 13 subarea-result"," ) ;; infectious
      file-type ( word item 14 subarea-result"," ) ;; newly infected
      file-type ( word item 15 subarea-result"," ) ;; newly deceased
      file-type ( word item 16 subarea-result"," ) ;; R0
    ]
    )
    file-print ""
  ]
  file-flush
  file-close-all
;; exporting world (if not headless) and plots
  if not multi-run? [ export-world worldfile-name ]
  export-plot "distances" ( word directory-name "/distances.csv" )
  export-plot "Current state of the population 1" ( word directory-name "/history1.csv" )
  export-plot "Current state of the population 2" ( word directory-name "/history2.csv" )
  export-plot "Infections in last 24 hours" ( word directory-name "/infections.csv" )
  export-plot "Deaths in last 24 hours" ( word directory-name "/deaths.csv" )
  export-plot "R0 real / est" ( word directory-name "/R0.csv" )
  export-plot "kappa" ( word directory-name "/kappa.csv" )
  export-plot "whom-I-infected" ( word directory-name "/whom-I-infected.csv" )
  export-plot "whom-I-infected_t" ( word directory-name "/whom-I-infected-t.csv" )
  export-plot "norm actions" ( word directory-name "/norm actions.csv" )
  export-plot "med perc risk" ( word directory-name "/perc risk.csv" )
  export-plot "sal WEAR A MASK" ( word directory-name "/sal WEAR A MASK.csv" )
  export-plot "sal BREATHE FREELY" ( word directory-name "/sal BREATHE FREELY.csv" )
  export-plot "sal STAY AT HOME" ( word directory-name "/sal STAY AT HOME.csv" )
  export-plot "sal GO TO PARTY" ( word directory-name "/sal GO TO PARTY.csv" )
  export-plot "test positivity rate" ( word directory-name "/test positivity rate.csv" )
  export-plot "virus infectiosity over time" ( word directory-name "/virus evolution.csv" )
  export-plot "moves" ( word directory-name "/moves.csv" )
  file-close-all
;; exporting the profiler results (if not headless)
  if not multi-run?
  [
    profiler:stop
    file-open profile-name
    file-print profiler:report
    file-close-all
    profiler:reset
  ]
  file-open logfile-name
;; writing an overview of all input parameters from the GUI
  util-flog-observer -2 "Input parameters:"
  util-flog-observer -2 ( word "asymptomatic-per-infected: " asymptomatic-per-infected )
  util-flog-observer -2 ( word "auto-lockdown            : " auto-lockdown )
  util-flog-observer -2 ( word "BACKGROUND               : " BACKGROUND )
  util-flog-observer -2 ( word "case-fatality-rate       : " case-fatality-rate )
  util-flog-observer -2 ( word "constant-seed            : " constant-seed )
  util-flog-observer -2 ( word "setup-seed               : " setup-seed )
  util-flog-observer -2 ( word "go-seed                  : " go-seed )
  util-flog-observer -2 ( word "distance-for-infection   : " distance-for-infection )
  util-flog-observer -2 ( word "far-commuter             : " far-commuter )
  util-flog-observer -2 ( word "heterogeneity            : " heterogeneity )
  util-flog-observer -2 ( word "illness-duration-max     : " illness-duration-max )
  util-flog-observer -2 ( word "illness-duration-min     : " illness-duration-min )
  util-flog-observer -2 ( word "infection-distribution   : " infection-distribution )
  util-flog-observer -2 ( word "infectiosity-duration    : " infectiosity-duration )
  util-flog-observer -2 ( word "immunity-duration-max    : " immunity-duration-max )
  util-flog-observer -2 ( word "immunity-duration-min    : " immunity-duration-min )
  util-flog-observer -2 ( word "incubation-time-max      : " incubation-time-max )
  util-flog-observer -2 ( word "incubation-time-min      : " incubation-time-min )
  util-flog-observer -2 ( word "infections-per-contact   : " infections-per-contact )
  util-flog-observer -2 ( word "infectious-from-abroad   : " infectious-from-abroad )
  util-flog-observer -2 ( word "initial-subareas         : " initial-subareas )
  util-flog-observer -2 ( word "interruptible?           : " interruptible? )
  util-flog-observer -2 ( word "lockdown-at              : " lockdown-at )
  util-flog-observer -2 ( word "lockdown-end-at          : " lockdown-end-at )
  util-flog-observer -2 ( word "long-distance            : " long-distance )
  util-flog-observer -2 ( word "max-months               : " max-months )
  util-flog-observer -2 ( word "move-speed               : " move-speed )
  util-flog-observer -2 ( word "mutation-rate            : " mutation-rate )
  util-flog-observer -2 ( word "NDW                      : " NDW )
  util-flog-observer -2 ( word "reporting-delay          : " reporting-delay )
  util-flog-observer -2 ( word "reporting-rate           : " reporting-rate )
  util-flog-observer -2 ( word "short-distance           : " short-distance )
  util-flog-observer -2 ( word "superspread?             : " superspread? )
  util-flog-observer -2 ( word "tested-per-infected      : " tested-per-infected )
  util-flog-observer -2 ( word "time-till-immunity-max   : " time-till-immunity-max )
  util-flog-observer -2 ( word "time-till-immunity-min   : " time-till-immunity-min )
  util-flog-observer -2 ( word "time-to-infectiosity-max : " time-to-infectiosity-max )
  util-flog-observer -2 ( word "time-to-infectiosity-min : " time-to-infectiosity-min )
  util-flog-observer -2 ( word "total-population         : " total-population )
  util-flog-observer -2 ( word "travellers               : " travellers )
  util-flog-observer -2 ( word "zero-person              : " zero-person )
  util-flog-observer -2 ( word "vacc-lasts-days          : " vacc-lasts-days )
  util-flog-observer -2 ( word "vaccination?             : " vaccination? )
  util-flog-observer -2 ( word "%continuous              : " %continuous )
;; writing the final state of output metrics to ./log.txt
  util-flog-observer -2 "Output metrics"
  util-flog-observer -2 ( word "end-of-run? is " end-of-run? ", stopdate: " stopdate ", day before: " day-before-stopdate )
  util-flog-observer -2 ( word "model stopped at " TNOW " the calendar length is " time:size-of-schedule ", real time is " date-and-time " " rectify-date )
  util-flog-observer -2 ( word "still susceptible    : " are-susceptible )
  util-flog-observer -2 ( word "infected             : " are-infected )
  util-flog-observer -2 ( word "ill                  : " are-ill )
  util-flog-observer -2 ( word "recovered            : " are-recovered )
  util-flog-observer -2 ( word "immune               : " are-immune )
  util-flog-observer -2 ( word "dead                 : " are-dead )
  util-flog-observer -2 ( word "infectious           : " are-infectious )
  util-flog-observer -2 ( word "never met infectious : " never-met )
  util-flog-observer -2 ( word "escaped infection    : " escaped )
  util-flog-observer -2 ( word "internally infected  : " infected )
  util-flog-observer -2 ( word "returned infected    : " injected )
  util-flog-observer -2 ( word "no more infectious at: " no-more-infectious-at )
  util-flog-observer -2 ( word "median co ca su      : " median-ccs )
  util-flog-observer -2 ( word "median co ca in      : " median-ccns )
  util-flog-observer -2 ( word "wear a mask          : " wear-a-mask )
  util-flog-observer -2 ( word "stay at home         : " stay-at-home )
  util-flog-observer -2 ( word "mean distance        : " mean-distance-to-nearest-neighbour )
  util-flog-observer -2 ( word "sdev distance        : " sdev-distance-to-nearest-neighbour )
  util-flog-observer -2 ( word "when-max-infected    : " when-max-infected " = " ( when-max-infected / 24 ) " days)" )
  util-flog-observer -2 ( word "n-of-max-infected    : " n-of-max-infected )
  util-flog-observer -2 ( word "when-max-deceased    : " when-max-deceased " = " ( when-max-deceased / 24 ) " days)" )
  util-flog-observer -2 ( word "n-of-max-deceased    : " n-of-max-deceased )
  util-flog-observer -2 ( word "mean-daily-moves     : " mean-daily-moves )
  util-flog-observer -2 ( word "vaccinated               : " vaccinated )
  util-flog-observer -2 ( word "vaccination-refused      : " vaccination-refused )
  util-flog-observer -2 "Transition Matrix:"
  util-flog-observer -2 "             [   sus    inf    ill    rec immune   dead]"
  ( foreach state-names transition-matrix
    [ [ stn line ] -> util-flog-observer -2 ( word ( substring ( insert-item 0 "         " stn ) 0 11 )": " map [ n -> bformat 6 n ] line ) ]
  )
  let stop-time rectify-date
  util-flog-observer -2 ( word "run lasted from " start-time " to " stop-time ", i.e. " precision run-length 3 " seconds, " precision ( run-length / 3600 ) 3 " hours")
end

;; collecting subarea statistics
to prepare-results
  if results = 0 [ set results [] ]
  let this-row []
  foreach n-values initial-subareas [ n -> n ]
  [ subarea-nr ->
    ask subarea subarea-nr
    [
      let my-R0 mean [ length whom-I-infected ] of my-people
      set this-row lput ( list T TNOW who color colour area density population
                          count my-people with [ state = "susceptible" ]
                          count my-people with [ state = "infected" ]
                          count my-people with [ state = "ill" ]
                          count my-people with [ state = "recovered" ]
                          count my-people with [ state = "immune" ]
                          count my-people with [ state = "dead" ]
                          count my-people with [ state = infectious? ]
                          my-newly-infected
                          my-newly-deceased
                          my-R0
                          my-incidence-rate
                        ) this-row
    ]
  ]
  set results lput this-row results
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; some shorthand procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; just shorthand what has to be done when an agent is tested positive
to report-a-positive-test
  set past-events-counter past-events-counter + 1
  set newly-infected-reported newly-infected-reported + 1
  util-flog 9 ( word "reported positive test, tests today " newly-infected-reported )
end

;; just shorthand what has to be done when an agent dies
to report-a-death
  set past-events-counter past-events-counter + 1
  set newly-deceased-reported newly-deceased-reported + 1
end

;; just shorthand for a move of an infectious to some distant patch, only used in user interactions
to move-an-infectious-far-away
  if any? infectious-people [ ask one-of infectious-people [ setxy random-xcor random-ycor ] ]
end

;; simulating the delayed report because of weekend,evening and night, used in the next two preocedures
to-report time-of-report
  let delay random-float-in-range 0.5 2.0
  let result ( time:plus T TNOW delay "hours" )
  let out-of-office-delays? random-float 1.0 < reporting-delay
  if out-of-office-delays?
  [
    let workday? false
    let office-hours? false
    let day-of-week time:get "dayofweek" T TNOW - 1
    if day-of-week < 5 [ set workday? true ]
    let time-of-day time:get "hour" T TNOW
    if time-of-day > 7 and time-of-day < 18 [ set office-hours? true ]
    if workday? and office-hours? [ set result ( time:plus  T TNOW ( random-float 2.0 ) "hours" ) ]
    if day-of-week < 4 and not office-hours?
    [
      ifelse time-of-day < 8
      [ set delay 9 - time-of-day ]
      [ set delay 31 - time-of-day ] set result ( time:plus T TNOW delay "hours" )
    ]
    if day-of-week = 4 and not office-hours?
    [ ifelse time-of-day < 8 [ set delay 9 - time-of-day ]
    [ set delay 48 + 31 - time-of-day ] set result ( time:plus T TNOW delay "hours" ) ]
    if day-of-week = 5 [ set delay 57 - time-of-day set result ( time:plus T TNOW delay "hours" ) ]
    if day-of-week = 6 [ set delay 33 - time-of-day set result ( time:plus T TNOW delay "hours" ) ]
  ]
  report result
end

;; report the result of an infectiosity test with a delay defined in time-of-report
to report-tested
  let this-time-of-report time-of-report
  util-flog 9 ( "report-tested entered" )
  time-schedule-event self [ [] -> report-a-positive-test ] "report a positive test" this-time-of-report
  util-flog 9 ( word "was tested as infected by " infection-chain " at " item ( time:get "dayofweek" T TNOW - 1 ) day-names
    " " T TNOW " and will be reported to statistics at " T this-time-of-report )
end

;; report a death with a delay defined in time-of-report
to report-death
  let this-time-of-report time-of-report
  time-schedule-event self [ [] -> report-a-death ] "report a death" this-time-of-report
  util-flog 15 ( word "passed away as infected by " infection-chain " and will be reported to statistics at " T this-time-of-report )
end

;; collecting all measures taken when a state changes

to update-transition-matrix [ old-state new-state ]
  util-flog 2 ( word "transition from " item old-state state-names " (" old-state ") to " item new-state state-names "( " new-state ")" )
  let old-state-vector item old-state transition-matrix
  let value item new-state old-state-vector
  set old-state-vector replace-item new-state old-state-vector ( value + 1 )
  set transition-matrix replace-item old-state transition-matrix old-state-vector
end

to set-state [ a-state ]
  let old-state position state state-names
  let new-state position a-state state-names
  ifelse a-state = "immune" or state = "recovered"
  [ util-flog 29 ( word "transition from " state " (" old-state ") to " a-state " (" new-state ")" ) ]
  [ util-flog 29 ( word "transition from " state " (" old-state ") to " a-state " (" new-state ")" ) ]
  ifelse state = "dead" and a-state != "dead"
  [
    util-flog 15 ( word "is already dead, change to " a-state " is impossible " )
  ]
  [
    ifelse state = a-state
    [
      util-flog 15 ( word "is already " state ", state cannot be changed to " a-state )
    ]
    [
      if a-state = "susceptible" [ set color white set shape "face happy" set infectious? false set size 0.6 ]
      if a-state = "infected" [ set color red ]
      if a-state = "ill" [set color violet set-infectious set past-events-counter past-events-counter - 1 ]
      if a-state = "recovered" [ set color blue set shape "face happy" set size 0.6 ]
      if a-state = "immune" [ set color green set shape "face happy" set infectious? false set size 0.6 ]
      if a-state = "dead" [ set color black set shape "Cross" set infectious? false set size 2 ]
      set state a-state
      update-transition-matrix old-state new-state
      let this-state a-state
      if a-state = "infected" [ set this-state ( word "infected by " my-potential-infector ) ]
      set state-history lput ( list this-state T TNOW patch-here size shape color ) state-history
    ]
  ]
  ifelse infectious? [ set shape "default" ] [ set shape state-shape ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; procedures called at the beginning of each new simulated day
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to allow-interruptions      ;; allows a user interaction at the end of each day, enabling users to stop, interrupt or manage the simulation run
  set user-halt? false
  let answer user-one-of "what to do?"
  [
    "continue"
    "go on forever"
    "print profile and continue forever"
    "halt and print profile and result!"
    "spread"
    "normal time"
    "lockdown"
    "inspect"
  ]
  if first answer = "c" [ set interruptible? true ]
  if first answer = "g" [ set interruptible? false user-message "No further intermediate stop" ]
  if first answer = "p" [ file-close-all file-open profile-name file-print profiler:report file-close-all set interruptible? false ]
  if first answer = "h"
  [
    file-close-all
    file-open profile-name
    file-print profiler:report
    file-close-all
    report-final-results
    show "halted intentionally by user"
    util-flog-observer -2 "run stopped intentionally by user"
    set user-halt? true
  ]
  if first answer = "s" [ move-an-infectious-far-away set interruptible? false ]
  if first answer = "n" [ set period "normal time" set interruptible? false if not multi-run? [ output-print ( word T TNOW ": user lockdown ended" ) ] ]
  if first answer = "l" [ set period "lockdown" set interruptible? false if not multi-run? [ output-print ( word T TNOW ": user lockdown" ) ] ]
  if first answer = "i" [ inspect one-of infectious-people set interruptible? false ]
end

;; performs a test on two five per cent samples of the total population
to perform-daily-test
  let n-of-tested-persons total-population / 10
  let random-test-persons n-of n-of-tested-persons people
  let suspect-test-persons min-n-of n-of-tested-persons people [ distance-to-my-potential-infector ]
  set random-positive-test-rate ( count random-test-persons with [ infectious? ] ) / n-of-tested-persons
  set suspect-positive-test-rate ( count suspect-test-persons with [ infectious? ] ) / n-of-tested-persons
  if random-positive-test-rate > max-positive-test-rate [ set max-positive-test-rate precision ( ceiling ( random-positive-test-rate * 1000 ) / 1000 ) 3 ]
  if suspect-positive-test-rate > max-positive-test-rate [ set max-positive-test-rate precision ( ceiling ( suspect-positive-test-rate * 1000 ) / 1000 ) 3 ]
end

;; doing some statistics to make the procedure daily-update more easily legible
to do-some-statistics
  if max [ future-moves ] of people > 1
  [
    if not multi-run? [ ask people with [ future-moves > 1 ] [ output-show ( word future-moves  " " state-history ) ] ]
    set ERROR-CONDITION? true
    error "max future-moves > 1"
  ]
  let day-of-week time:get "dayofweek" T TNOW - 1
  set weekend? day-of-week > 4
  if day-of-week = 5
  [ ask people with [ state = "susceptible" or state = "infected" or state = "immune" or state = "recovered" ] [ go-home ] ]
  set past-events-counter past-events-counter + 1
;; the maximum frequency of the distributions of the folowing variables is only calculated once a week as it is quite time consuming
  if day-of-week < 5
  [
    set max-freq-ccs max-freq 20 [ conviviality * carelessness ] of people with [ state = "susceptible" ]
    set max-freq-cci max-freq 20 [ conviviality * carelessness ] of people with [ state != "susceptible" ]
    set max-freq-ccp max-freq 20 [ my-risk ] of people
    set max-freq-pir max-freq 20 [ perceived-risk ] of people
    set max-freq-vis max-freq 20 [ virus-infectiosity ] of people with [ state != "susceptible" ]
    set max-freq-vii max-freq 20 [ virus-infectiosity ] of people with [ infectious? ]
    let mode-norm-saliences []
    foreach [ 0 1 2 3 ] [ mns -> set mode-norm-saliences lput ( max-freq 20 [ item mns my-norm-saliences ] of people ) mode-norm-saliences ]
    set max-freq-ns max mode-norm-saliences
  ]
;; moving averages are not calculated before the desired number of values is reached
  ifelse length newly-deceased-7-days = 7
  [
    set newly-deceased-7-days fput newly-deceased but-last newly-deceased-7-days
    set newly-deceased-7-days-ma mean newly-deceased-7-days
    if newly-deceased-7-days-ma > item 1 newly-deceased-7-days-ma-max [ set newly-deceased-7-days-ma-max ( list ticks newly-deceased-7-days-ma ) ]
  ]
  [
    set newly-deceased-7-days fput newly-deceased newly-deceased-7-days
  ]
  ifelse length newly-infected-7-days = 7
  [
    set newly-infected-7-days fput newly-infected but-last newly-infected-7-days
    set newly-infected-7-days-ma mean newly-infected-7-days
    if newly-infected-7-days-ma > item 1 newly-infected-7-days-ma-max [ set newly-infected-7-days-ma-max ( list ticks newly-infected-7-days-ma ) ]
    ifelse length newly-infected-7-days-ma-5-days = 5
    [
      set newly-infected-7-days-ma-5-days fput newly-infected-7-days-ma but-last newly-infected-7-days-ma-5-days
    ]
    [
      set newly-infected-7-days-ma-5-days fput newly-infected-7-days-ma newly-infected-7-days-ma-5-days
    ]
  ]
  [
    set newly-infected-7-days fput newly-infected newly-infected-7-days
  ]
  ifelse length newly-infected-4-days = 4
  [
    set newly-infected-4-days fput newly-infected but-last newly-infected-4-days
    set newly-infected-4-days-ma mean newly-infected-4-days
    ifelse length newly-infected-4-days-ma-5-days = 5
    [
      set newly-infected-4-days-ma-5-days fput newly-infected-4-days-ma but-last newly-infected-4-days-ma-5-days
    ]
    [
      set newly-infected-4-days-ma-5-days fput newly-infected-4-days-ma newly-infected-4-days-ma-5-days
    ]
  ]
  [
    set newly-infected-4-days fput newly-infected newly-infected-4-days
  ]
;; finds out whether the number of new infections increases or decreases
  ifelse empty? newly-infected-7-days-ma-5-days
  [ set pandemic-grows? true ]
  [ set pandemic-grows? first newly-infected-7-days-ma-5-days > last newly-infected-7-days-ma-5-days ]
  prepare-results
  set total-infectiosity-rate count infectious-people / count people
;; added 2020-10-25 to calculate the dispersion parameter (reproduction-number now is a global variable instead of a value calculated within a plot)
  set reproduction-number ifelse-value ( count infectious-people  > 0 ) [ mean [ length whom-I-infected ] of infectious-people ] [ 0 ]
  ifelse count infectious-people > 2
  [
    set var-infected variance [ length whom-I-infected ] of infectious-people
    set dispersion-parameter ifelse-value reproduction-number = 0 [ 0 ] [ 1 + ( var-infected - reproduction-number ) / sqr reproduction-number ]
  ]
  [ set dispersion-parameter 0 ]
;; inserted 2021-02-10 to improve R0 plot
  if reproduction-number > R0-max-ever [ set R0-max-ever reproduction-number ]
  if length newly-infected-4-days-ma-5-days = 5 and last newly-infected-4-days-ma-5-days > 0 [ set R04 first newly-infected-4-days-ma-5-days / last newly-infected-4-days-ma-5-days ]
  if length newly-infected-7-days-ma-5-days = 5 and last newly-infected-7-days-ma-5-days > 0 [ set R07 first newly-infected-7-days-ma-5-days / last newly-infected-7-days-ma-5-days ]
  if R04 > R0-max-ever [ set R0-max-ever R04 ]
  if R07 > R0-max-ever [ set R0-max-ever R07 ]
  set R0-max-ever ceiling ( R0-max-ever * 10 ) / 10
;; added 2021-01-24 to calculate output metrics
  set never-met count people with [ met-infectious-not-infected = 0  and state = "susceptible" ]
  set escaped  count people with [ met-infectious-not-infected > 0 and state = "susceptible" ]
  set infected count people with [ state != "susceptible" and not member? first first first state-history "zf" ]
  set injected count people with [ state != "susceptible" and member? first first first state-history  "zf" ]
  set are-susceptible count people with [ state = "susceptible" ]
  set are-infected    count people with [ state = "infected" ]
  set are-ill         count people with [ state = "ill" ]
  set are-recovered   count people with [ state = "recovered" ]
  set are-immune      count people with [ state = "immune" ]
  set are-dead        count people with [ state = "dead" ]
  set are-infectious  count people with [ infectious? ]
  set median-ccs      median [ conviviality * carelessness ] of people with [ state = "susceptible" ]
  set median-ccns     median [ conviviality * carelessness ] of people with [ state != "susceptible" ]
  set wear-a-mask     count people with [ wears-a-mask? ]
  set stay-at-home    count people with [ stays-at-home? ]
  set when-max-infected first newly-infected-7-days-ma-max
  set n-of-max-infected last newly-infected-7-days-ma-max
  set when-max-deceased first newly-deceased-7-days-ma-max
  set n-of-max-deceased last newly-deceased-7-days-ma-max
end

;; the following three procedures collect all measures taken at the onset   and at the end of lockdown and quarantine, respectively
to enter-lockdown
  if not multi-run? [ output-print ( word T TNOW ": " period " to lockdown " incidence-rate ) ]
  util-flog-observer 2 ( word T TNOW ": " period " to lockdown " incidence-rate )
  ifelse
  period = "normal time" ;; i.e. when lockdown is first entered
  [ ask people [ update-explicit-compliance-invocations "NORM: WEAR A MASK" ] ]
  [ ask people [ update-explicit-violations "NORM: WEAR A MASK" ] ] ;; i.e. when lockdown is left and normal times entered the compliance invocations are neutralised
  set period "lockdown"
end
to enter-quarantine
  set period "quarantine"  if not multi-run? [ output-print ( word T TNOW ": lockdown to quarantine  " incidence-rate ) ]
  util-flog-observer -2 ( word T TNOW ": lockdown to quarantine  " incidence-rate )
end
to leave-lockdown
  if not multi-run? [ output-print ( word T TNOW ": " period " to normal time " incidence-rate ) ]
  util-flog-observer -2 ( word T TNOW ": " period " to normal time " incidence-rate )
  set period "normal time"
end

;; updates a number of global and individual variables at the beginning of each simulated day
to daily-update
  set seconds-per-day timer
  set run-length run-length + seconds-per-day
  reset-timer
  set incidence-rate sum newly-infected-7-days * 100000 / total-population
  if auto-lockdown != "off"          ;; start and end an automatically defined lockdown when certain thresholds are passed
  [
    ( ifelse
      ( period = "quarantine" and incidence-rate < lockdown-at * 2 )
      [ enter-lockdown ]
      ( period = "lockdown" )
      [
        ( ifelse
          ( incidence-rate > lockdown-at * 4 ) [ enter-quarantine ]
          ( incidence-rate < lockdown-end-at ) [ leave-lockdown ]
        )
      ]
      ( period = "normal time" and incidence-rate > lockdown-at ) [ enter-lockdown ]
    )
  ]
  set infectious-people people with [ infectious? ]
  util-flog-observer -2 ( word "PAST DAY ENDED AFTER " seconds-per-day " SECONDS OF CALCULATION, total so far " run-length )
  util-flog-observer -2 ( word "A NEW DAY BEGINS " item ( time:get "dayofweek" T TNOW - 1 ) day-names " " T TNOW ", " count infectious-people " people infectious, incidence rate " precision incidence-rate 1 ", real time is " date-and-time " " rectify-date )
  do-some-statistics
;; line added 2020-10-25 to keep track of the calculation of reproduction number, its variance and the dispersion parameter
  ifelse any? infectious-people and var-infected > 0 and var-infected != reproduction-number
  [
    let p 1 - reproduction-number / var-infected
    let r sqr reproduction-number / ( var-infected - reproduction-number )
    let freq-list []
    foreach n-values 10 [ n -> n ]  [ nfl -> set freq-list lput precision ((  count infectious-people with [ length whom-I-infected = nfl ] ) / count infectious-people ) 3 freq-list ]
    set max-freq-wii max freq-list * count people with [ infectious? ]
    util-flog-observer -2 ( word "R0 = " precision reproduction-number 3 ", variance = " precision var-infected 3 ", p = " precision p 3 ", r = " precision r 3 ", dispersion parameter = " precision dispersion-parameter 3 ", frequencies = " freq-list ", " precision sum freq-list 3 )
  ]
  [ util-flog-observer -2 "no infectious agents left or var-infected = 0 or var-infected = reproduction-number" ]
;; agents calculate their perceived risk of meeting an infectious person and/or the global risk of being infected
  ask people with [ state = "susceptible" ]
  [
    let neighbours people in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour )
    set locally-perceived-risk ( count neighbours with [ infectious? ] / count neighbours )
    ifelse pandemic-grows? or period = "lockdown" or period = "quarantine"
    [ set perceived-risk max list locally-perceived-risk total-infectiosity-rate ]
    [ set perceived-risk min list locally-perceived-risk total-infectiosity-rate ]
  ]
  perform-daily-test
  update-plots
  ask subareas
  [
    let my-color color + 6
    ifelse length my-newly-infected-7-days = 7
    [
      set my-newly-infected-7-days lput my-newly-infected but-first my-newly-infected-7-days
      set my-incidence-rate sum my-newly-infected-7-days / count my-people * 100000
      if my-incidence-rate >  50 and my-incidence-rate <=  100 [ ask my-patches [ set pcolor my-color - 0.3 ] ]
      if my-incidence-rate > 100 and my-incidence-rate <=  200 [ ask my-patches [ set pcolor my-color - 0.6 ] ]
      if my-incidence-rate > 200 and my-incidence-rate <=  400 [ ask my-patches [ set pcolor my-color - 0.9 ] ]
      if my-incidence-rate > 400 and my-incidence-rate <=  800 [ ask my-patches [ set pcolor my-color - 1.2 ] ]
      if my-incidence-rate > 800 and my-incidence-rate <= 1600 [ ask my-patches [ set pcolor my-color - 1.5 ] ]
      if my-incidence-rate > 1600                              [ ask my-patches [ set pcolor my-color - 1.8 ] ]
    ]
    [ set my-newly-infected-7-days lput my-newly-infected my-newly-infected-7-days ]
    set my-newly-infected 0
    set my-newly-deceased 0
  ]
;; global variables holding the number of events of the day are reset
  set newly-deceased-reported 0
  set newly-deceased 0
  set newly-infected-reported 0
  set newly-infected 0
;; determining whether the run can be ended when there are no more infectious agents
;; slightly changed to let the model run after there were three months without infectious people
  set end-of-wave? end-of-wave? or not any? infectious-people
  if end-of-wave? and no-more-infectious-at = 0 [ set no-more-infectious-at ticks ]
  if end-of-wave? and any? infectious-people [ set end-of-wave? false util-flog-observer -2 ( word "old wave ended at " no-more-infectious-at ", new wave starts" ) ]
  time-schedule-days ( time:plus TNOW 1 "days" )
  let too-slow? seconds-per-day > 420
  let stoptime-reached? time:is-equal? T TNOW day-before-stopdate or time:is-after? T TNOW day-before-stopdate
  let waiting-time-for-new-infections 24
  if infectious-from-abroad != "none" [ set waiting-time-for-new-infections 2160 ]                                 ;; if no infectious from abroad, the run stops one day later
  let no-more-infectious? ( end-of-wave? and ( ticks - no-more-infectious-at ) > waiting-time-for-new-infections ) ;; this condition is only true when there were no infectious people for 90 days
  set end-of-run? end-of-run? or too-slow? or user-halt? or ERROR-CONDITION? or no-more-infectious? or stoptime-reached?
  util-flog-observer -2 ( word "end-of-run? " end-of-run? ", too-slow? " too-slow? ", user-halt? " user-halt? ", ERROR-CONDITION? " ERROR-CONDITION?
                               ", no-more-infectious? " no-more-infectious? ", stoptime-reached? " stoptime-reached? )
  if end-of-run? and no-more-infectious-at = 0
  [
    ifelse not any? people with [ infectious? ]
    [
      set no-more-infectious-at ticks
      util-flog-observer -2 ( word "end of run, no more infectious at " no-more-infectious-at " " T TNOW " (day " ( precision ( ticks / 24 ) 3 ) ")" )
    ]
    [
      set no-more-infectious-at -1
      util-flog-observer -2 ( word "stopdate reached, still " count people with [ infectious? ] " left" )
    ]
  ]
  if not end-of-run? and not multi-run?  ;; a last opportunity for user interaction
  [
    if interruptible? [ allow-interruptions ]
  ]
  if not multi-run?     ;; except in behaviour space or headless the state of the interface of the day is sent to a file
  [
    export-interface ( word directory-name "/" format 5 ( ticks / 24 ) ".png" )
  ]
  ifelse end-of-run?
  [
    util-flog-observer -2 ( word "schedule length is " time:size-of-schedule " and will now be cleared" )
    time:clear-schedule
    if too-slow? [ util-flog-observer -2 ( word "run stopped because the simulation cost " seconds-per-day " per day." ) ]
    if no-more-infectious? [ util-flog-observer -2 ( word "end-of-run? is " end-of-run? ", current schedule is " time:show-schedule ", no more infectious at " no-more-infectious-at ) ]
    if ERROR-CONDITION? [ util-flog-observer -2 "an error occurred" ]
    if user-halt? [ util-flog-observer -2 "halted by user, output will be written" ]
    if stoptime-reached? [ util-flog-observer -2 "stop time reached" ]
  ]
  [   ;; otherwise some information about the state of the run is written to ./log.txt
    util-flog-observer -2 ( word "end-of-run? is " end-of-run? ", after daily update at " T TNOW " the calendar length is " time:size-of-schedule )
  ]
  set mean-daily-moves daily-moves / total-population
  set daily-moves 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; added 2021-08-04 to enable vaccination
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  if days > vaccination-start
  [
    set doses-available doses-available + round ( total-population / vacc-lasts-days  )
  ]
  display
end



;; two shorthands for random times within intervals
to-report incubation-time
  report random-float-in-range incubation-time-min incubation-time-max
end

to-report illness-duration
  report random-float-in-range illness-duration-min illness-duration-max
end

;; calculates the propensities to take each of the possible actions,
;; updates the memory of an agent and
;; sends messages to neighbours (asks neighbours to update their respective memory slots) --- redone 2021-01-11
to update-normative-board
  set my-neighbours other people in-radius 5
  if count my-neighbours > LISTENERS
  [
    set my-neighbours min-n-of LISTENERS my-neighbours [ distance myself ]
;;    show sort [ precision distance myself 4 ] of my-neighbours              ;; was used for debugging
  ]
  set my-party-propensity propensity-to "GOING TO PARTY"
  set my-mask-propensity propensity-to "PUTTING ON MASK"
  set my-unmask-propensity propensity-to "PUTTING OFF MASK"
  set my-stay-propensity propensity-to "STAYING AT HOME"
  let i'll-unmask false
  let i'll-party false
  let i'll-mask false
  let i'll-stay false
  set i'll-party stays-at-home? and random-float 1.0 < my-party-propensity
  set i'll-unmask wears-a-mask? and random-float 1.0 < my-unmask-propensity
  set i'll-mask not wears-a-mask? and random-float 1.0 < my-mask-propensity
  set i'll-stay not stays-at-home? and random-float 1.0 < my-stay-propensity
  if member? self witnesses
  [
    util-flog 17 ( word "party " precision my-party-propensity 3 ", unmask " precision my-unmask-propensity 3 ", stay " precision my-stay-propensity 3 ", mask " precision my-mask-propensity 3
                   ", my risk: " precision my-risk 3
                   " " ifelse-value stays-at-home? [ "sits at home" ] [ "went to work or party" ] " / " ifelse-value wears-a-mask? [ "wears-a-mask" ] [ "breathes freely" ] )
    util-flog 17 ( word "party " i'll-party ", unmask " i'll-unmask ", stay " i'll-stay ", mask " i'll-stay )
  ]
  if i'll-unmask
  [
    set wears-a-mask? false
    update-compliances "NORM: BREATHE FREELY"
    update-violations "NORM: WEAR A MASK"
    if any? my-neighbours
    [
      ask my-neighbours
      [
        update-observed-compliances "NORM: BREATHE FREELY"
        update-observed-violations "NORM: WEAR A MASK"
      ]
    ]
  ]
  if i'll-mask
  [
    set wears-a-mask? true
    update-compliances "NORM: WEAR A MASK"
    update-violations "NORM: BREATHE FREELY"
    if any? my-neighbours
    [
      ask my-neighbours
      [
        update-observed-compliances "NORM: WEAR A MASK"
        update-observed-violations "NORM: BREATHE FREELY"
      ]
    ]
  ]
  if i'll-party
  [
    set stays-at-home? false
    update-compliances "NORM: GO TO PARTY"
    update-violations "NORM: STAY AT HOME"
    if any? my-neighbours
    [
      ask my-neighbours
      [
        update-observed-compliances "NORM: GO TO PARTY"
        update-observed-violations "NORM: STAY AT HOME"
      ]
    ]
  ]
  if i'll-stay
  [
    set stays-at-home? true
    update-compliances "NORM: STAY AT HOME"
    update-violations "NORM: GO TO PARTY"
    if any? my-neighbours
    [
      ask my-neighbours
      [
        update-observed-compliances "NORM: STAY AT HOME"
        update-observed-violations "NORM: GO TO PARTY"
      ]
    ]
  ]
end

;; calculates the propensity to take a given action
to-report propensity-to [ action ]
  if not member? action all-actions
  [
    util-flog 5 ( word "propensity-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
    report -1
  ]
  if member? self witnesses
  [
    util-flog 17 ( word "calculating propensity to " action )
    util-flog 17 ( word  ": C: " MP my-compliances " V: "  MP my-violations  " OC: "  MP other-compliances " OV: "  MP other-violations
      " P: " MP punishments  " S: "  MP sanctions " EC: "  MP explicit-compliance-invocations " EV: "  MP explicit-violations
      " SAL: " MP my-norm-saliences )
  ]
  let individual-drive calculate-my-individual-drive-to action
  let normative-drive calculate-my-normative-drive-to action
  let a-prop ( IDW * individual-drive + NDW * normative-drive )
  if a-prop > 1 or a-prop < 0
  [
    util-flog (-2) ( word "ERROR: a-prop out of bounds: " a-prop " = " IDW "*" individual-drive " + " NDW "*" normative-drive )
    set ERROR-CONDITION? true
    error "ERROR: a-prop out of bounds: "
  ]
  if member? self witnesses
  [ util-flog 17 ( word  ": ID = " precision individual-drive 4 ", NF = " precision normative-drive 4 ", propensity to " action " = " precision a-prop 4 ) ]
  report a-prop
end

;; calculates the normative drive to take aspecial action, used in the preceding procedure
to-report calculate-my-normative-drive-to [ action ]
  let normative-drive 0
  if not member? action all-actions
  [
    util-flog 0 ( word "calculate-my-normative-drive-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
  ]
  if action = "GOING TO PARTY"
  [
    calculate-a-salience "NORM: GO TO PARTY"
    calculate-a-salience "NORM: STAY AT HOME"
    let np-dont position "NORM: STAY AT HOME" my-norms
    let np-do position "NORM: GO TO PARTY" my-norms
    let normative-drive-to-go-to-party item np-do my-norm-saliences
    let normative-drive-not-to-go-to-party item np-dont my-norm-saliences
    ifelse normative-drive-to-go-to-party > normative-drive-not-to-go-to-party
    [
      set normative-drive normative-drive-to-go-to-party
    ]
    [
      set normative-drive 1 - normative-drive-not-to-go-to-party
    ]
  ]
  if action = "STAYING AT HOME"
  [
    calculate-a-salience "NORM: STAY AT HOME"
    calculate-a-salience "NORM: GO TO PARTY"
    let np-do position "NORM: STAY AT HOME" my-norms
    let np-dont position "NORM: GO TO PARTY" my-norms
    let normative-drive-to-stay-at-home item np-do my-norm-saliences
    let normative-drive-not-to-stay-at-home item np-dont my-norm-saliences
    ifelse normative-drive-to-stay-at-home > normative-drive-not-to-stay-at-home
    [
      set normative-drive normative-drive-to-stay-at-home
    ]
    [
      set normative-drive 1 - normative-drive-not-to-stay-at-home
    ]
  ]
  if action = "PUTTING ON MASK"
  [
    calculate-a-salience "NORM: WEAR A MASK"
    calculate-a-salience "NORM: BREATHE FREELY"
    let np-do position "NORM: WEAR A MASK" my-norms
    let np-dont position "NORM: BREATHE FREELY" my-norms
    let normative-drive-to-put-on-mask item np-do my-norm-saliences
    let normative-drive-to-put-off-mask item np-dont my-norm-saliences
    ifelse normative-drive-to-put-on-mask > normative-drive-to-put-off-mask
    [
      set normative-drive normative-drive-to-put-on-mask
    ]
    [
      set normative-drive 1 - normative-drive-to-put-off-mask
    ]
  ]
  if action = "PUTTING OFF MASK"
  [
    calculate-a-salience "NORM: WEAR A MASK"
    calculate-a-salience "NORM: BREATHE FREELY"
    let np-do position "NORM: BREATHE FREELY" my-norms
    let np-dont position "NORM: WEAR A MASK" my-norms
    let normative-drive-to-put-off-mask item np-do my-norm-saliences
    let normative-drive-to-put-on-mask item np-dont my-norm-saliences
    ifelse normative-drive-to-put-off-mask > normative-drive-to-put-on-mask
    [
      set normative-drive normative-drive-to-put-off-mask
    ]
    [
      set normative-drive 1 - normative-drive-to-put-on-mask
    ]
  ]
  report normative-drive
end


;; calculates the individual drive to take a special action, used in the procedure propensity-to, new version as of 2021-01-19
to-report calculate-my-individual-drive-to [ action ]
;; the agents's individual drive to take any action depends only on the estimated probability of being infected
  let my-old-risk my-risk
  set my-risk max ( list ( my-old-risk * 0.9 ) ( conviviality * carelessness * perceived-risk ) )
;; hence my-risk is composed as follows:
;; conviviality is my propensity to join crowds
;; carelessness is my propensity to be careless
;; perceived-risk is the probability of meeting infectious people
;; the product of these three terms is the probability of being infected
;; if this estimated probability is lower than it was at the time of the previous calculation it is at least 90 percent of the previous value
  let individual-drive 0
  if not member? action all-actions
  [
    util-flog 0 ( word "calculate-my-individual-drive-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
  ]
  ( ifelse
    action = "PUTTING ON MASK" [ set individual-drive my-risk ]
    action = "STAYING AT HOME" [ set individual-drive 0.5 * my-risk ]
    action = "PUTTING OFF MASK" [ set individual-drive ( 1 - my-risk ) ]
    action = "GOING TO PARTY" [ set individual-drive ( 1 - 0.5 * my-risk ) ]
  )
  report individual-drive
end

;; calculates the salience of a norm from the contents of the agent's normative board (its normative memory)
;; the salience of a norm is calculated with a formula explained in the ODD (Appendix "Minding Norms in a Nutshell")
;; as this formula cannot be appropriately represented in this program code, its main components are commented below
to calculate-a-salience [ a-norm ]
  if not member? a-norm all-norms
  [
    util-flog 0 ( word "calculate-a-salience: " a-norm " is not a valid norm." )
    set ERROR-CONDITION? true
    stop
  ]
  let np position a-norm my-norms
;; C, V, OC, OV, P, S, EC and EV contain the numbers of the different kinds of norm invocations stored in
;; the normative memory of an agent- The differences of positive and negative invocations, divided by their
;; sums, are weighted with the coefficients WC .. WE. First each of the terms (C-V)/(C+V) etc. have to be
;; calculated. In parallel two constants have to be calculated (calle beta and one-divided-by-alpha) which
;; make sure that the formula yields a value between 0 and 1
  let C item np my-compliances
  let V item np my-violations
  let OC item np other-compliances
  let OV item np other-violations
  let P item np punishments
  let S item np sanctions
  let EC item np explicit-compliance-invocations
  let EV item np explicit-violations
  let beta 0
  let one-divided-by-alpha 0
;; the first term OWN=(C-V)/(C+V)
  let OWN 0
  if C + V > 0
  [
    set OWN ( C - V ) / ( C + V )
  ]
  set beta beta + WC
  set one-divided-by-alpha one-divided-by-alpha + 2 * WC
;; the second term OBS=(OC-OV)/(OC+OV)
  let OBS 0
  if OC + OV > 0
  [
    set OBS ( OC - OV ) / ( OC + OV )
  ]
  set beta beta + WO
  set one-divided-by-alpha one-divided-by-alpha + 2 * WO
;; the third term NPV=(OV+V-P-S)/(OV+V)
  let NPV 0
  if OV + V > 0
  [
    set NPV (OV + V - P - S )
    if NPV < 0 [ set NPV 0 ]
    set NPV NPV / ( OV + V)
  ]
  set beta beta + WNPV
  set one-divided-by-alpha one-divided-by-alpha + WNPV
;; the first part of the fourth term P_max=P/max(OV+V,P+S)
  let maxPSOV P + S
  if OV + V > P + S [ set maxPSOV OV + V ]
  let P_max 0
  if maxPSOV > 0
  [
    set P_max P / maxPSOV
  ]
  set one-divided-by-alpha one-divided-by-alpha + WP
;; the second part of the fourth term S_max/max(OV+V,P+S)
  let S_max 0
  if maxPSOV > 0
  [
    set S_max S / maxPSOV
  ]
  set one-divided-by-alpha one-divided-by-alpha + WS
;; the fifth term ECEV=(EC-EV)/(EC+EV)
  let ECEV 0
  if EC + EV > 0
  [
    set ECEV ( EC - EV ) / ( EC + EV )
  ]
  set beta beta + WE
  set one-divided-by-alpha one-divided-by-alpha + 2 * WE
;; calculate the weighted sum of the five terms
  let SAL 0
  if one-divided-by-alpha > 0
  [
    set SAL ( ( beta + OWN * WC + OBS * WO - NPV * WNPV + P_max * WP + S_max * WS + ECEV * WE ) / one-divided-by-alpha )
  ]
;; and checking whether the result is between 0 and 1
  if SAL < 0 or SAL > 1
  [
    util-flog -2 ( word "ERROR? " a-norm ": " ( list C V OC OV P S EC EV ) )
    util-flog -2 ( word "ERROR? " a-norm ": " ( list OWN OBS NPV P_max S_max ECEV beta one-divided-by-alpha SAL ) )
    set ERROR-CONDITION? true
  ]
  if member? self witnesses [ util-flog 20 ( word  ": salience of " a-norm " calculated as " precision SAL 4 ) ]
;; finally insert the calculated salience into at the respective place in the list of norm saliences for further use
  set my-norm-saliences replace-item np my-norm-saliences SAL
end

;; all kinds of agents update their individual event board (called by every update-... procedure)
to increase-invocation-counter [ i-item a-norm ]
  let this-norm position a-norm all-norms
  let this-sublist item this-norm invocation-count
  let this-subsublist item 1 this-sublist
  let this-entry item i-item this-subsublist
  set this-subsublist replace-item i-item this-subsublist ( this-entry + 1 )
  set this-sublist replace-item 1 this-sublist this-subsublist
  set invocation-count replace-item this-norm invocation-count this-sublist
end

;; the following procedures update the different fields of the normative board
to update-compliances [ a-norm ]
  increase-invocation-counter 0 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np my-compliances
      set my-compliances replace-item np my-compliances ( old-value * DISCOUNT + 1 )
  ]
  ask my-neighbours [ update-observed-compliances a-norm ]
end

to update-violations [ a-norm ]
  increase-invocation-counter 1 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np my-violations
      set my-violations replace-item np my-violations ( old-value * DISCOUNT + 1 )
  ]
  ask my-neighbours [ update-observed-violations a-norm ]
end

to update-observed-compliances [ a-norm ]
  increase-invocation-counter 2 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np other-compliances
      set other-compliances replace-item np other-compliances ( old-value * DISCOUNT + 1 )
  ]
end

to update-observed-violations [ a-norm ]
  increase-invocation-counter 3 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np other-violations
      set other-violations replace-item np other-violations ( old-value * DISCOUNT + 1 )
  ]
end

to update-punishments [ a-norm ]
  increase-invocation-counter 4 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np punishments
    set punishments replace-item np punishments ( old-value * DISCOUNT + 1 )
  ]
end

to update-sanctions [ a-norm ]
  increase-invocation-counter 5 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np sanctions
    set sanctions replace-item np sanctions ( old-value * DISCOUNT + 1 )
  ]
end

to update-explicit-compliance-invocations [ a-norm ]
  increase-invocation-counter 6 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np explicit-compliance-invocations
    set explicit-compliance-invocations replace-item np explicit-compliance-invocations ( old-value * DISCOUNT + 1 )
  ]
end

to update-explicit-violations [ a-norm ]
  increase-invocation-counter 7 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np explicit-violations
    set explicit-violations replace-item np explicit-violations ( old-value * DISCOUNT + 1 )
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; utilities
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; converts a number into a string with leading zeroes of the desired number of digits, used for setting the names of the directories for the run results
to-report format [ digits number ]
  let result ( word round number )
  while [ length result < digits ] [ set result ( word "0" result ) ]
  report result
end

to-report bformat [ digits number ]
  let result ( word round number )
  while [ length result < digits ] [ set result ( word " " result ) ]
  report result
end


;; to assign a shape according to health state
to-report state-shape
  let result "face neutral"
  if state = "dead" [ set result "Cross" ]
  if state = "susceptible" or state = "recovered" or state = "immune" [ set result "face happy" ]
  if state = "infected" or state = "ill" [ set result "face sad" ]
  report result
end


;; a wrapper for a time procedure, avoiding possible error situations
to time-schedule-days [ a-time ]
  if ( time:is-between? a-time TNOW stopdate ) and not ( time:is-equal? a-time TNOW ) and not end-of-run?
  [
    time:schedule-event "observer" [ [] -> daily-update ] a-time
    util-flog-observer 20 ( word "; observer; daily-update;" T a-time )
  ]
end

;; a wrapper for a time procedure, avoiding possible error situations
to time-schedule-event [ executor a-task a-task-name a-time  ]
  if a-task-name = "report a positive test" [ util-flog 99 ( word ( precision ( ticks / 24 ) 3 ) "; " executor "; " state " " a-task-name "; scheduled for " T a-time ) ]
  if ( time:is-between? a-time TNOW stopdate ) and not ( time:is-equal? a-time TNOW ) and not end-of-run?
  [
    ifelse ( a-task-name = "move around" and future-moves = 0 )
    [
      time:schedule-event executor a-task a-time
      set future-moves future-moves + 1
      set prospective-move ( list ticks state a-time patch-here )
    ]
    [ time:schedule-event executor a-task a-time ]
    if a-task-name = "lose immunity" [ set immunity-loss-scheduled? true ]
    ifelse state = "recovered" or a-task-name = "become immune"
    [ util-flog 29 ( word ( precision ( ticks / 24 ) 3 ) "; " executor "; " state " " a-task-name "; scheduled for " T a-time ) ]
    [ util-flog 29 ( word ( precision ( ticks / 24 ) 3 ) "; " executor "; " state " " a-task-name "; scheduled for " T a-time ) ]
    if a-task-name = "report a positive test" [ util-flog 99 ( word ( precision ( ticks / 24 ) 3 ) "; " executor "; " state " " a-task-name "; scheduled for " T a-time ) ]
  ]
end

;; just a shorthand for the precision function to make the output more easily legible
to-report MP [ value-list ]
  report map [ value -> precision value 1 ] value-list
end

;; implements a uniform distribution with arbitrary min and max
to-report random-float-in-range [ a-min a-max ]
  report a-min + random-float ( a-max - a-min )
end

;; implements the beta distribution
to-report random-beta [ p q ]
  let gamma1 random-gamma p 1
  let gamma2 random-gamma q 1
  let result gamma1 / ( gamma1 + gamma2 )
  report result
end

;; shorthand for the square of a number
to-report sqr [ a-number ]
  report a-number * a-number
end

;; reports the current logotime
to-report TNOW
  report time:plus startdate ticks "hours"
end

;; reports a string containing the current date and time
to-report T [ a-logotime ]
  ifelse a-logotime = NULL or a-logotime = 0
  [
    report a-logotime
  ]
  [
    report time:show a-logotime "yyyy-MM-dd HH:mm:ss.SSS"
  ]
end

;; allows the observer to write to the logfile
to util-flog-observer [ level something ]
  set days round ( ticks / 24 )
  file-open logfile-name
  if level < log-level [ file-show ( word "; day " days "; " level "; " something ) ]
  file-flush
end

;; allows an agent to write to the logfile
to util-flog [ level something ]
  file-open logfile-name
  if level < log-level or ( witness? and ( level < ( log-level - 30 ) ) ) [ file-show ( word "; " precision ( ticks / 24 ) 3 " days; " level "; " something ) ]
  file-flush
end

;; calculates the maximum height of a histogram
to-report max-freq [ ncat a-list ]
  if empty? a-list [ report 0.1 ]
  let a-table map [ value -> ( round ( value * ncat ) ) / ncat ] a-list
  let mode max modes a-table
  report reduce
    [ [ m n ] -> ifelse-value ( n = mode ) [ m + 1 ] [ m ] ] ( fput 0 a-table )
end

to-report rectify-date
  let date-string date-and-time
  let year substring date-string 23 27
  let month substring date-string 19 22
  if month = "Okt" [ set month "Oct" ]
  if month = "Dez" [ set month "Dec" ]
  if month = "Mär" [ set month "Mar" ]
  if month = "Mai" [ set month "May" ]
  let day substring date-string 16 18
  let AMPM substring date-string 13 15
  let hour substring date-string 0 2
  let mmin substring date-string 3 5
  let sec substring date-string 6 12
  let month-number ( 1 + ( position month ( list "Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec" ) ) )
  ifelse month-number < 10 [ set month ( word "0" month-number ) ] [ set month ( word month-number ) ]
  let num-hour read-from-string hour
  let num-min read-from-string mmin
  let num-sec read-from-string sec
  let time-of-day ( num-sec + 60 * num-min + 3600 * num-hour ) / 3600
  if ( AMPM = "PM" or AMPM = "pm" ) and time-of-day < 12 [ set hour ( word ( num-hour + 12 ) ) ]
  if ( AMPM = "AM" or AMPM = "am" ) and hour = "12" [ set hour "00" ]
  set date-string ( word year "-" month "-" day "-" hour "-" mmin "-" sec )
  report date-string
end
to-report rectify-date-test [ date-string ]
  let year substring date-string 23 27
  let month substring date-string 19 22
  if month = "Okt" [ set month "Oct" ]
  if month = "Dez" [ set month "Dec" ]
  if month = "Mär" [ set month "Mar" ]
  if month = "Mai" [ set month "May" ]
  let day substring date-string 16 18
  let AMPM substring date-string 13 15
  let hour substring date-string 0 2
  let mmin substring date-string 3 5
  let sec substring date-string 6 12
  let month-number ( 1 + ( position month ( list "Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec" ) ) )
  ifelse month-number < 10 [ set month ( word "0" month-number ) ] [ set month ( word month-number ) ]
  let num-hour read-from-string hour
  let num-min read-from-string mmin
  let num-sec read-from-string sec
  let time-of-day ( num-sec + 60 * num-min + 3600 * num-hour ) / 3600
  if ( AMPM = "PM" or AMPM = "pm" ) and time-of-day < 12 [ set hour ( word ( num-hour + 12 ) ) ]
  if ( AMPM = "AM" or AMPM = "am" ) and hour = "12" [ set hour "00" ]
  set date-string ( word year "-" month "-" day "-" hour "-" mmin "-" sec )
  report date-string
end
@#$#@#$#@
GRAPHICS-WINDOW
712
13
1507
809
-1
-1
7.8
1
5
1
1
1
0
0
0
1
-50
50
-50
50
1
1
1
ticks
30.0

BUTTON
10
10
90
45
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
95
10
175
45
go
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

SLIDER
195
55
354
88
initial-subareas
initial-subareas
2
13
13.0
1
1
NIL
HORIZONTAL

SLIDER
365
55
545
88
max-months
max-months
0
24
24.0
1
1
NIL
HORIZONTAL

SLIDER
548
415
705
448
lockdown-at
lockdown-at
0
200
100.0
10
1
NIL
HORIZONTAL

SLIDER
548
473
708
506
lockdown-end-at%
lockdown-end-at%
0
100
70.0
5
1
NIL
HORIZONTAL

SLIDER
14
90
194
123
far-commuter
far-commuter
0
1
0.65
0.05
1
NIL
HORIZONTAL

SLIDER
194
90
351
123
short-distance
short-distance
0
10
8.0
1
1
NIL
HORIZONTAL

SLIDER
364
93
544
126
long-distance
long-distance
0
100
80.0
1
1
NIL
HORIZONTAL

SLIDER
9
170
259
203
distance-for-infection
distance-for-infection
0
5
0.5
0.05
1
NIL
HORIZONTAL

SLIDER
12
215
206
248
infections-per-contact
infections-per-contact
0
1
0.05
0.05
1
NIL
HORIZONTAL

SLIDER
355
129
543
162
tested-per-infected
tested-per-infected
0
1
1.0
0.05
1
NIL
HORIZONTAL

SLIDER
210
217
437
250
asymptomatic-per-infected
asymptomatic-per-infected
0
1
0.65
0.05
1
NIL
HORIZONTAL

SLIDER
377
359
539
392
case-fatality-rate
case-fatality-rate
0
1
0.04
0.01
1
NIL
HORIZONTAL

SLIDER
9
253
189
286
time-to-infectiosity-min
time-to-infectiosity-min
0
6
0.0
1
1
NIL
HORIZONTAL

SLIDER
189
253
369
286
time-to-infectiosity-max
time-to-infectiosity-max
time-to-infectiosity-min
15
4.0
1
1
NIL
HORIZONTAL

SLIDER
12
292
192
325
incubation-time-min
incubation-time-min
0
20
2.0
1
1
NIL
HORIZONTAL

SLIDER
192
292
372
325
incubation-time-max
incubation-time-max
incubation-time-min
20
20.0
1
1
NIL
HORIZONTAL

SLIDER
12
327
192
360
illness-duration-min
illness-duration-min
0
30
12.0
1
1
NIL
HORIZONTAL

SLIDER
192
327
372
360
illness-duration-max
illness-duration-max
illness-duration-min
60
30.0
1
1
NIL
HORIZONTAL

SLIDER
12
357
192
390
time-till-immunity-min
time-till-immunity-min
0
20
0.0
1
1
NIL
HORIZONTAL

SLIDER
192
357
372
390
time-till-immunity-max
time-till-immunity-max
time-till-immunity-min
30
2.0
1
1
NIL
HORIZONTAL

SLIDER
12
392
192
425
immunity-duration-min
immunity-duration-min
0
24
6.0
1
1
NIL
HORIZONTAL

SLIDER
192
392
372
425
immunity-duration-max
immunity-duration-max
immunity-duration-min
72
12.0
3
1
NIL
HORIZONTAL

CHOOSER
459
169
556
214
zero-person
zero-person
"densest" "sparsest" "center" "random"
1

SWITCH
10
828
190
861
interruptible?
interruptible?
1
1
-1000

PLOT
1522
13
1912
177
Current state of the population 1
time (days)
pcm
0.0
10.0
0.0
1000.0
true
true
"" "set-plot-y-range 0 100000"
PENS
"susceptible" 1.0 0 -7500403 true "" "plotxy round days are-susceptible * 100000 / total-population"
"immune" 1.0 0 -13840069 true "" "plotxy days are-immune * 100000 / total-population"
"all-ever-infected" 1.0 0 -6459832 true "" "plotxy days all-ever-infected * 100000 / total-population"

PLOT
1530
173
1874
333
Current state of the population 2
time (days)
pcm
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"infected" 1.0 0 -2674135 true "" "plotxy days are-infected * 100000 / total-population"
"ill" 1.0 0 -10141563 true "" "plotxy days are-ill * 100000 / total-population"
"recovered" 1.0 0 -11221820 true "" "plotxy days are-recovered * 100000 / total-population"
"dead" 1.0 0 -16777216 true "" "plotxy days are-dead * 100000 / total-population"
"infectious" 1.0 0 -955883 true "" "plotxy days are-infectious * 1000 / total-population"

PLOT
1530
333
1866
493
Infections in last 24 hours
time (days)
pcm
0.0
10.0
0.0
0.1
true
true
"" ""
PENS
"real" 1.0 0 -16777216 true "" "plotxy days newly-infected * 100000 / total-population"
"reported" 1.0 0 -2674135 true "" "plotxy days newly-infected-reported * 100000 / total-population"
"mov aver" 1.0 0 -11221820 true "" "if length newly-infected-7-days = 7 [ plotxy ( ticks / 24 - 3 ) newly-infected-7-days-ma  * 100000 / total-population ]"

PLOT
1530
493
1866
653
Deaths in last 24 hours
time (days)
pcm
0.0
10.0
0.0
0.1
true
true
"" ""
PENS
"real" 1.0 0 -16777216 true "" "plotxy days newly-deceased  * 100000 / total-population"
"reported" 1.0 0 -2674135 true "" "plotxy days newly-deceased-reported * 100000 / total-population"
"mov aver" 1.0 0 -11221820 true "" "if length newly-deceased-7-days = 7 [ plotxy ( ticks / 24 - 3 ) newly-deceased-7-days-ma * 100000 / total-population ]"

PLOT
1530
653
1735
813
R0 real / est
time (days)
R0
0.0
1.0
0.0
7.0
true
true
"" "set-plot-y-range 0 R0-max-ever"
PENS
"R0 real" 1.0 0 -16777216 true "" "if are-infectious > 0 [ plotxy days reproduction-number ]"
"1.0" 1.0 0 -955883 true "" "plotxy days 1.0"
"R0 est 4" 1.0 0 -13345367 true "" "plotxy days R04 "
"R0 est 7" 1.0 0 -13791810 true "" "plotxy days R07"

MONITOR
177
10
362
55
Date and time
( word item ( time:get \"dayofweek\" T TNOW - 1 ) day-names \" \" T TNOW )
17
1
11

MONITOR
9
967
2302
1016
subarea statistics (area density population incidence)
sort-by [ [?1 ?2] -> item 0 ?1 < item 0 ?2 ] [ ( list who substring colour 0 3 area precision density 2 population round my-incidence-rate ) ] of subareas
17
1
12

MONITOR
1918
13
2008
50
susceptible
count people with [ state = \"susceptible\" ]
17
1
9

MONITOR
1918
53
2008
90
infected
count people with [ state = \"infected\" ]
17
1
9

MONITOR
1918
93
2008
130
ill
count people with [ state = \"ill\" ]
17
1
9

MONITOR
1918
133
2008
170
recovered
count people with [ state = \"recovered\" ]
17
1
9

MONITOR
1918
173
2008
210
immune
count people with [ state = \"immune\" ]
17
1
9

MONITOR
1918
213
2008
250
dead
count people with [ state = \"dead\" ]
17
1
9

MONITOR
2008
13
2098
50
inf/sus
count people with [ state = \"susceptible\" and infectious? ]
17
1
9

MONITOR
2008
53
2098
90
inf/inf
count people with [ state = \"infected\" and infectious? ]
17
1
9

MONITOR
2008
93
2098
130
inf/ill
count people with [ state = \"ill\" and infectious? ]
17
1
9

MONITOR
2008
133
2098
170
inf/rec
count people with [ state = \"recovered\" and infectious? ]
17
1
9

MONITOR
2008
173
2098
210
inf/imm
count people with [ state = \"immune\" and infectious? ]
17
1
9

MONITOR
2008
213
2098
250
inf/dead
count people with [ state = \"dead\" and infectious? ]
17
1
9

MONITOR
2008
253
2098
290
infectious
count people with [ infectious? ]
17
1
9

MONITOR
167
920
309
969
scheduled events
time:size-of-schedule
17
1
12

MONITOR
369
10
553
55
stop date
( word item ( time:get \"dayofweek\" T day-before-stopdate - 1 ) day-names \" \" T day-before-stopdate )
17
1
11

MONITOR
1892
658
1947
703
R0(t)
reproduction-number
3
1
11

MONITOR
282
869
471
918
NIL
newly-infected-7-days
17
1
12

MONITOR
282
919
472
968
NIL
newly-deceased-7-days
17
1
12

MONITOR
400
427
546
472
NIL
period
17
1
11

MONITOR
1874
337
1949
374
new infected
newly-infected
0
1
9

MONITOR
1874
369
1949
406
reported
newly-infected-reported
17
1
9

MONITOR
1874
403
1949
440
mov average
newly-infected-7-days-ma
2
1
9

MONITOR
1919
253
2009
290
all ever
all-ever-infected
0
1
9

OUTPUT
7
472
547
654
9

MONITOR
1874
495
1949
532
new deceased
newly-deceased
17
1
9

MONITOR
1874
528
1949
565
reported
newly-deceased-reported
17
1
9

MONITOR
1874
560
1949
597
mov average
newly-deceased-7-days-ma
2
1
9

TEXTBOX
553
392
716
418
reported infections pcm pop- ulation during past seven days
9
0.0
1

SLIDER
188
128
352
161
reporting-delay
reporting-delay
0
1
0.0
0.05
1
NIL
HORIZONTAL

PLOT
9
705
186
825
conviv*carel su
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state = \"susceptible\" ] [ set-plot-y-range 0 max-freq-ccs ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state = \"susceptible\" ] [ set-plot-y-range 0 max-freq-ccs ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ conviviality * carelessness ] of people with [ state = \"susceptible\" ]"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ conviviality * carelessness ] of people with [ state = \"susceptible\" ] 0 plot-pen-down plotxy median [ conviviality * carelessness ] of people with [ state = \"susceptible\" ] max-freq-ccs ]"

PLOT
185
705
354
825
conviv*carel in
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state != \"susceptible\" ] [ set-plot-y-range 0 max-freq-cci ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state != \"susceptible\" ] [ set-plot-y-range 0 max-freq-cci ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ conviviality * carelessness ] of people with [ state != \"susceptible\" ]"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ conviviality * carelessness ] of people with [ state != \"susceptible\" ] 0 plot-pen-down plotxy median [ conviviality * carelessness ] of people with [ state != \"susceptible\" ] max-freq-cci ]"

PLOT
362
705
542
825
conviv*carel*perc
NIL
NIL
0.0
0.4
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-ccp  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-ccp  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ my-risk ] of people"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ my-risk ] of people max-freq-ccp  ]"

PLOT
542
705
702
825
perc inf risk
NIL
NIL
0.0
0.4
0.0
19.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ perceived-risk ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-pir ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ perceived-risk ] of people"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ perceived-risk ] of people 0 plot-pen-down plotxy median [ perceived-risk ] of people max-freq-pir ]"

MONITOR
169
870
283
919
past events
past-events-counter
17
1
12

MONITOR
610
870
690
915
max
max [ perceived-risk ] of people
4
1
11

MONITOR
1892
700
2068
745
R0 MA 4
map [ ?1 -> precision ?1 1 ] newly-infected-4-days-ma-5-days
17
1
11

MONITOR
1892
748
2069
793
R0 MA 7
map [ ?1 -> precision ?1 1 ] newly-infected-7-days-ma-5-days
17
1
11

PLOT
693
844
853
964
med perc risk
time
NIL
0.0
10.0
0.0
0.001
true
false
"" "set-plot-y-range 0 ifelse-value any? people with [ perceived-risk > 0.001 ][ precision ( max [ perceived-risk ] of people ) 3 ] [ 0.001 ]"
PENS
"default" 1.0 0 -16777216 true "" "if any? people [ plotxy days median [ perceived-risk ] of people ]"
"pen-1" 1.0 0 -7500403 true "" "if any? people [ plotxy days max [ perceived-risk ] of people ]"

MONITOR
9
869
134
914
NIL
pandemic-grows?
17
1
11

MONITOR
8
918
162
967
moves
( word min [ future-moves ] of people \" | \" count people with [ future-moves < 0 ] \" / \" count people with [ future-moves = 0 ] \" / \" count people with [ future-moves > 0 ] \" | \" max [ future-moves ] of people)
17
1
12

MONITOR
610
915
689
960
median
median [ perceived-risk ] of people
4
1
11

MONITOR
7
659
70
704
median
median-ccs
4
1
11

MONITOR
185
659
248
704
median
median-ccns
4
1
11

MONITOR
480
659
543
704
median
median [ my-risk ] of people
4
1
11

MONITOR
639
658
702
703
median
median [ perceived-risk ] of people
4
1
11

SLIDER
14
128
186
161
reporting-rate
reporting-rate
0
1
1.0
0.05
1
NIL
HORIZONTAL

CHOOSER
544
512
708
557
infectious-from-abroad
infectious-from-abroad
"none" "holidays" "continuous" "seasonal" "months 2..11" "months 7..8" "months 9..11" "months 2..3 6..9"
3

SLIDER
182
433
315
466
NDW
NDW
0
1
0.9
0.05
1
NIL
HORIZONTAL

SLIDER
9
433
181
466
BACKGROUND
BACKGROUND
-10
10
-6.0
1
1
NIL
HORIZONTAL

MONITOR
2105
747
2200
792
mask wearers
count people with [ wears-a-mask? ]
17
1
11

MONITOR
2200
747
2312
792
staying at home
count people with [ stays-at-home? ]
17
1
11

PLOT
2107
14
2307
134
sal WEAR A MASK
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ item 0 my-norm-saliences ] of people"

PLOT
2107
133
2307
253
sal BREATHE FREELY
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ item 1 my-norm-saliences ] of people"

PLOT
2108
253
2308
373
sal STAY AT HOME
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ item 2 my-norm-saliences ] of people"

PLOT
2109
372
2309
492
sal GO TO PARTY
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ item 3 my-norm-saliences ] of people"

PLOT
2065
793
2312
973
norm actions
time (days)
pcm
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"mask" 1.0 0 -13791810 true "" "plotxy days wear-a-mask / total-population * 100000"
"at home" 1.0 0 -13840069 true "" "plotxy days stay-at-home / total-population * 100000"

PLOT
1860
794
2060
944
whom-I-infected
NIL
NIL
0.0
4.0
0.0
1.0
true
false
"set-plot-pen-mode 1\nset-plot-pen-interval 0.2" "set-plot-pen-mode 1\nset-plot-x-range 0 ifelse-value any? people with [ infectious? and length whom-I-infected > 0 ] [ max ( list 2.5 max [ length whom-I-infected ] of people with [ infectious? ] ) ] [ 2.5 ]\nset-plot-y-range 0 max (list 0.1 round max-freq-wii )"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ length whom-I-infected ] of people with [ infectious? ]"

MONITOR
264
170
457
215
mean / sdev near neighb dist
( list precision mean-distance-to-nearest-neighbour 5 precision sdev-distance-to-nearest-neighbour 5 )
6
1
11

CHOOSER
193
827
332
872
heterogeneity
heterogeneity
"none" "wrt risk" "wrt norm" "both"
1

MONITOR
873
868
1006
913
max daily infected
map [ ?1 -> precision ?1 1 ] ( replace-item 0 newly-infected-7-days-ma-max (( item 0 newly-infected-7-days-ma-max ) / 24 ))
1
1
11

MONITOR
1009
868
1122
913
max daily dead
map [ ?1 -> precision ?1 1 ] replace-item 0 newly-deceased-7-days-ma-max ( ( item 0 newly-deceased-7-days-ma-max ) / 24 )
17
1
11

PLOT
1532
817
1859
967
test positivity rate
NIL
NIL
0.0
10.0
0.0
0.001
true
true
"" "set-plot-y-range 0 max ( list 0.001 max-positive-test-rate )"
PENS
"random" 1.0 0 -16777216 true "" "plotxy days random-positive-test-rate"
"suspect" 1.0 0 -2674135 true "" "plotxy days suspect-positive-test-rate"

SLIDER
10
55
193
88
total-population
total-population
0
100000
38149.0
2000
1
NIL
HORIZONTAL

MONITOR
2007
659
2072
704
1+1/k(t)
dispersion-parameter
3
1
11

MONITOR
1947
659
2005
704
var
var-infected
17
1
11

PLOT
1329
818
1529
968
whom-I-infected_t
NIL
NIL
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"< 1" 1.0 0 -16777216 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 1 ] / count infectious-people ) ]"
"< 2" 1.0 0 -7500403 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 2 ] / count infectious-people ) ]"
"< 3" 1.0 0 -2674135 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 3 ] / count infectious-people ) ]"
"< 4" 1.0 0 -955883 true "" "if any? infectious-people [ plotxy  days ( count infectious-people with [ length whom-I-infected < 4 ] / count infectious-people ) ]"
"< 5" 1.0 0 -6459832 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 5 ] / count infectious-people ) ]"
"< 6" 1.0 0 -16777216 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 6 ] / count infectious-people ) ]"
"< 7" 1.0 0 -7500403 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 7 ] / count infectious-people ) ]"
"< 8" 1.0 0 -2674135 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 8 ] / count infectious-people ) ]"
"< 9" 1.0 0 -955883 true "" "if any? infectious-people [ plotxy  days ( count infectious-people with [ length whom-I-infected < 9 ] / count infectious-people ) ]"
"all" 1.0 0 -6459832 true "" "if any? infectious-people [ plotxy days ( count infectious-people with [ length whom-I-infected < 100 ] / count infectious-people ) ]"

SWITCH
554
625
707
658
superspread?
superspread?
1
1
-1000

SLIDER
595
555
707
588
travellers
travellers
0
0.1
0.015
0.001
1
NIL
HORIZONTAL

MONITOR
874
919
973
964
NIL
no-more-infectious-at
17
1
11

PLOT
1130
820
1330
970
distances
NIL
NIL
0.0
0.01
0.0
2.0
true
false
"set-plot-pen-mode 1\nset-plot-pen-interval 0.05\nset-plot-x-range 0 3" "set-plot-pen-mode 1\nset-plot-pen-interval 0.05\nset-plot-x-range 0 3\n"
PENS
"default" 1.0 0 -16777216 true "" "histogram distances "

MONITOR
1877
442
1972
487
7 days incidence
precision incidence-rate 1
17
1
11

MONITOR
977
919
1061
964
speed
seconds-per-day
17
1
11

PLOT
2074
627
2312
747
virus infectiosity infectious
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-plot-pen-interval 0.05\nset-plot-y-range 0 max-freq-vii" "set-plot-pen-mode 1\nset-plot-pen-interval 0.05\nset-plot-y-range 0 max-freq-vii\nset-plot-x-range 0 1"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ virus-infectiosity ] of people with [ infectious? ]"

SLIDER
442
218
589
251
mutation-rate
mutation-rate
0
0.1
0.05
0.01
1
NIL
HORIZONTAL

PLOT
2074
493
2313
630
virus infectiosity over time
NIL
NIL
0.0
1.0
0.5
1.0
true
false
"" ""
PENS
"mean" 1.0 0 -16777216 true "" "if any? people with [ infectious?] [ plotxy days mean [ virus-infectiosity ] of people with [ infectious? ] ]"
"median" 1.0 0 -7500403 true "" "if any? people with [ infectious?] [ plotxy days median [ virus-infectiosity ] of people with [ infectious? ] ]"
"min" 1.0 0 -2674135 true "" "if any? people with [ infectious?] [ plotxy days min [ virus-infectiosity ] of people with [ infectious? ] ]"
"max" 1.0 0 -10899396 true "" "if any? people with [ infectious?] [ plotxy days max [ virus-infectiosity ] of people with [ infectious? ] ]"

MONITOR
854
815
912
860
asym
mean [ virus-asymptomaticity ] of people with [ state != \"susceptible\" ]
17
1
11

MONITOR
917
817
975
862
infe
mean [ virus-infectiosity ] of people with [ infectious? ]
17
1
11

CHOOSER
373
254
537
299
infection-distribution
infection-distribution
"uniform" "beta 2 5" "beta 5 2"
2

MONITOR
1064
919
1122
964
NIL
days
17
1
11

PLOT
1953
293
2113
443
missed inf
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"set-plot-pen-mode 1" ""
PENS
"default" 1.0 0 -16777216 true "" "histogram [ met-infectious-not-infected ] of people with [ met-infectious-not-infected > 0 ]"

MONITOR
2003
473
2070
518
never met
never-met
17
1
11

MONITOR
2003
518
2073
563
escaped
escaped
17
1
11

MONITOR
2004
563
2069
608
infected
infected
17
1
11

MONITOR
2004
608
2069
653
injected
injected
17
1
11

SLIDER
378
394
540
427
infectiosity-duration
infectiosity-duration
0
30
24.0
1
1
NIL
HORIZONTAL

SLIDER
560
13
708
46
move-speed
move-speed
0.05
10
2.4
0.05
1
NIL
HORIZONTAL

MONITOR
620
174
708
219
moves p.d.
mean-daily-moves
2
1
11

MONITOR
977
818
1125
863
R0 real / est
( list precision reproduction-number 3 precision R04 3 precision R07 3 )
3
1
11

PLOT
1735
652
1895
815
kappa
NIL
NIL
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -13840069 true "" "ifelse ( are-infectious > 1 ) [ plotxy days dispersion-parameter ][ plotxy (ticks / 24 ) 0 ]"

SLIDER
595
589
709
622
%continuous
%continuous
0
100
30.0
5
1
NIL
HORIZONTAL

PLOT
550
52
710
172
moves
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-plot-pen-interval 0.1" "set-plot-pen-mode 1\nset-plot-pen-interval 0.1\nif days > 0 and max [ my-moves ] of people > 0\n[\n  set-plot-x-range 0 precision ( max [ my-moves / days ] of people ) 2\n  set-plot-y-range 0 max ( list 1 ( count people with [ my-moves / days < 0.1 ] ) ) \n]"
PENS
"default" 1.0 0 -16777216 true "" "if days > 0 [ histogram [ my-moves / days ] of people ]"

INPUTBOX
472
865
611
925
setup-seed
-1.941496835E9
1
0
Number

CHOOSER
478
830
636
875
constant-seed
constant-seed
"all-random" "setup-constant" "all-constant"
2

INPUTBOX
478
927
606
987
go-seed
-1.25165254E9
1
0
Number

CHOOSER
549
344
687
389
auto-lockdown
auto-lockdown
"off" "internal" "external"
0

TEXTBOX
557
447
707
473
percentage of lockdown-at\nto end lockdown
10
0.0
1

SWITCH
549
299
695
332
vaccination?
vaccination?
0
1
-1000

MONITOR
550
254
710
299
doses / served / refused
( list doses-available vaccinated vaccination-refused )
17
1
11

SLIDER
395
305
552
338
vacc-lasts-days
vacc-lasts-days
180
720
720.0
30
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

This is an event-oriented model of an epidemic in which agents move around, eventually infect each other, fall ill, recover, become immune or die from the disease. Agents sense how many infectious other agents are in their neighbourhood and all over the population and calculate their personal risk of being infected depending on their own (constant) conviviality and carelessness and in their (variable) risk perception,

In the norm-oriented version the also communicate about

  * wearing masks (as opposed to breathe freely) which reduces their probability of being infected by the factor 0.2 (can be changed in the code) and
  * staying at home (instead of going to places where they can be infected) which reduces their probability of being infected to 0.

Switching between either behaviours corroborates slightly the propensity to keep on with the new behaviour; neighbours are encouraged to behave in the same manner.

## HOW IT WORKS

The model covers an area of 10,201 patches, divided in several (up to 13) regions (`subareas`) with the same or different population density.

When the model starts, exactly one agent in the subarea with the highest density (alternatively: with the smallest density, or at a random patch) is infected.

At the very beginning all agents are scheduled to move a short distance within some short time. If at the end of this move they encounter an infectiuos agent they encounter the risk of getting infected,  continue to move around, become infectious themselves, fall ill with a certain probability, go to hospital and after they recovered or died return to their start patch. If they recovered they continue to move, lose their infectiosity after some time, become immune for a limited (but rather long) period, and if they lose their immunity they become susceptive again.

Infections are reported with some delay, but at the very latest when they fall ill and are brought to hospital.

The individual risk of being infected depends, of course, on how many infectious agents are in the neighbourhood (defined by `distance-for-infection`, but also on three attributes of the individual agents:

  * `conviviality`, a beta distributed instance variable between 0 and 1 initialised in the setup and kept constant
  * `carelessness` , another beta distributed instance variable between 0 and 1 initialised in the setup and kept constant
  * `perceived-risk` which at any time is either the percentage of infectious agents within a radius of 5 (while the epidemic increases) or the overall percentage of infectious agents (otherwise).

`conviviality' and 'carelessness` can alternatively be kept identical for all people when the `heterogeneity` switch is set to `both` or `wrt risk`.

The risk of being infected is proportional to the product of the global constant `infections-per-contact`, and the individual attributes `conviviality`, `carelessness` and `( 1 - perceived-risk )`.

If the normative mode is switched on, all agents are initialised with a memory of past norm invocations (for the theory behind norm invocations and norm-dependent behaviour see Conte et al. 2014 and Elsenbroich et al. 2016). The number of such norm invocations depends on the global variable `BACKGROUND` (-10 .. +10) which determines the mean of the number of invocations of norms relating to a fear of the pandemic and the fear of losing one's freedom (the individual numbers are either equal to BACKGROUND or beta distributed around this mean with a range of 20, depending on the `heterogeneity` parameter). The norms are the following:

  * `FEAR OF PANDEMIC` related:
    * `NORM: STAY AT HOME`
    * `NORM: WEAR A MASK`
  * `FEAR OF OPPRESSION` related:
    * `NORM: GO TO PARTY`
    * `NORM: PUT OFF MASK`

Agents calculate the salience of these norm and the propensity to take repsective actions:

  * `PUTTING ON MASK`
  * `PUTTING OFF MASK`
  * `STAYING AT HOME`
  * `GOING TO PARTY`

The respective propensity depends on both the _individual drive_ and the _normative drive_. The former depends only on the perceived risk (and is the same as in the non-normative mode), whereas the latter depends on the current salience of the four norms.

Plots and monitors keep track of the epidemic. All of them are described in more detail in the ODD+D description accompanying this model.

The R0 plot keeps track of three variants of the _basic reproduction number R<sub>0</sub>_; the "real" reproduction number is calculated as the mean of the length of the infection chain of all currently infectious agents, the two others are calculated according to the rules published by the German Robert Koch Institute, in two variants, one using 4-day moving averages, the other uses 7-day moving averages of daily new infections, _R<sub>0</sub>_ being calculated as the quotient of the current moving average and the moving average four days earlier, hence _MA<sub>t</sub>/MA<sub>t-4</sub>_. See https://www.rki.de/DE/Content/Infekt/EpidBull/Archiv/2020/Ausgaben/17_20.pdf, p. 10--16.

## HOW TO USE IT

The buttons, sliders, switches etc. have the following meaning (from top left to bottom right, left of the view):

  * `setup` and `go` have the usual meaning, but in an event-oriented model `go` is a _go forever_ and can either be stopped with Tools->Halt or via an interruption mechanism explained below.
  * _Date and time_ gives the simulated time which starts atthe beginning of the year 2020.
  * _stop date_ shows how long the simulation can run (namely for `max-months` months, if it is not interrupted before or ends when no infectious agents are left.
  * `total-population` sets the exact number of people spread over the subareas according to the `density-mode` parameter with equal or different densities among the subareas.
  * `initial-subareas` determines how many subregions (`subareas`) are displayed.
  * `density-mode` allows for three different modes of populating the `subareas`: all with the same population density, with linearly decreasing density and with inversely decreasing as a function of the number of patches; a patch can sprout up to four agents — alternatively: nine agents — (except on the boundary of a neighbouring `subarea` where there is one or no agent per patch.
  * `infected-from-abroad` determines whether it is possible that some agents are infected from abroad (as if they had just returned from holidays abroad); in this case about two percent of the population become infected and infectious from the second to the eleventh simulated month (such an event is logged in the output area).
  * `period` shows whether the model is in lockdown mode or in normal mode; this can be switched automatically or by the user.
  * `max-months` determines the maximum duration of the simulation run.
  * `far-commuter` determines the percentage of agents which can move up to `long-distance` patches at a time.
  * `short-distance` determines how far (at most) the ordinary agents can walk at a time.
  * `long-distance` determines how far (at most) the far-commuting agents can walk at a time.
  * `auto-lockdown` determines whether the model can enter the lockdown mode automatically. "Lockdown means" that the distances up to which agents may move are lowered to one half or its normal va,ue.
  * `lockdown-at` determines the limit for entering the lockdown: when the incidence rate (newly infectious agents in the paast seven days pcm) becomes greater than this limit, lockdown starts at the next midnight.
  * `lockdown-end-at` determines the limit for exiting the lockdown: when the incidence rate falls below this limit, lockdown ends at the next midnight.
  * `distance-for-infection` is the maximum distance within which infections are possible.
  * `infections-per-contact` is the ratio between successful infections and contacts.
  * `tested-per-infected` is the ratio between tested (and, hence, reportable) infections and all infections.
  * `asymptomatic-per-infected` is the ratio between the number of agents ehich show symptoms resulting in being hospitalised.
  * `case-fatality-rate` is the ratio between all deceased and all infected.
  * `time-to-infectiosity-min` and `-max` is the range of an uniform distribution for the time between being infected and being infectious (in days).
  * `incubation-time-min` and `-max`  is the range of an uniform distribution for the time between between being infected and showing symptoms of illness that lead to hospitalisation (in days).
  * 'illness-duration-min` and `-max` is the range of an uniform distribution for the time between falling ill and recovering or death (in days).
  * `immunity-duration-min` and `-max` is the range of an uniform distribution for the time between recovering and becoming susceptible after loss of immunity (in months).
  * `reporting-delay` is the upper end of the range of an uniform distribution determining when a test result is reported; over the weekend further delays are programmed (see code).
  * `zero-person` determines whether `person-zero` occurs in the `"densest"`, `"sparsest"` or a `"random" subarea`.
  * `interruptible?` determines whether the simulation run can be interrupted at the end of each simulated day. The switch can be changed at any time, and at midnight of the next simulated day a chooser opens which allows the user to determine how the simulation should go on. The options are
    * `"continue"` until next midnight;
    * `"go on forever"`; continue forever;
    * `"print profile and continue forever"`: (only for debugging purposes) print a profile (duration of all procedures) and continue;
    * `"halt and print profile and result!"`: end the simulation run and output all collected information;
    * `"spread"`:  `move-an-infectious-far-away`, i.e. select an infectious agent randomly and put it on a random patch;
    * `"normal time"`: exit from a user-defined lockdown;
    * `"lockdown"`: start a user-defined lockdown;
    * `"inspect"` a randomly selected infectious agent.

The followingng events can be scheduled by the model:

  * `move-around`: the agent moves to another place
  * `set-infectious`: the agent is infected and becomes infectious
  * `report-a-positive-test`: the agents is reported to the observer as infected
  * `fall-ill`: the agent shows symptoms and is brought to hospital
  * `recover-or-die`: the agent either recovers or dies and is returned to its original patch
  * `report-a-death`: the death of the agent is reported to the observer
  * `become-immune`: the agent is no longer infectious nor susceptible
  * `lose-immunity`: the agent loses its immunity and is susceptible again

The monitors and plots are more or less self-explanatory:

  * (left of the view) the two plots `conviv*careless...` show the distribution and the median of the product of the two individual constant attributes `conviviality` and `carelessness` separately for the susceptible and for all other agents and the individual variable attribute `perceived-risk` which is at the same time the probability of being infected once an infectious contact happened.
  *  the plot `conviv*carel*perc...` shows the distribution and the median of the product of the two individual constant attributes `conviviality` and `carelessness` and the individual variable attribute `perceived-risk` which is at the same time the probability of being infected once an infectious contact happened.
  * `perc inf risk` shows the distribution and the mean of the individual variable attribute `perceived-risk` which while the moving-average of the dayly number of newly infected agents rises is the percentage of infectious agents within a radius of 5, otherwise it is the overall percentage of infectious agents.
  * `mean and max perc risk` is the maximum of this distribution.
  * `med perc risk` represents the maximum and the median of the distribution of the `perceived-risk` attribute over time.
  * (right of the view) `current state of the population`: these two plots represent several global percentages,
    * the upper plot containing the percentages of susceptible agents and of all who were ever infected,
    * the second plot containing the percebtages of currently infected, infectious, ill, recovered, immune  and deceased persons
separated for better legibility.
  * `infectious in last 24 hours` represents the number of newly infected, those whose test was reported and a 7-day moving average of the former.
  * `deaths in past 24 hours` represents the number of newly deceased, those which were reported and a moving average of the former.
  * `R0 real / est` represents the daily estimates of the basic reproduction number (see above).
  * (below the view) the day and number when the daily infections and deaths,respectively, were at their maximum.
  * the current incidence rate.
  * the number,colour, area, density, population and the number of currently infectious agents per subarea.



## THINGS TO NOTICE

In each run, the model creates a directory into which 15 files are written (the final state of the world, the plots, a profile, a logging file with details about interesting events, and a result file with details about the epidemic histories in the individual subareas). At the end of every simulated day, a PNG file is written, and these PNG files can be converted to an AVI file using mencoder (NetLogo's movie feature is not used as not all video programmes can work with MOV files). The world and the PNG files are not written in behaviour space mode.

Which events are logged can be determined by changing the first parameter of the util-flog procedure and/or by setting the respective switch.


## THINGS TO TRY

Try out what happens with user lockdowns or with automatic lockdowns, and perhaps change the parameters determining what happens during a lockdown.

Change the algorithm which determines the different risk perception during periods of decreasing and increasing new infections, respectively.

Change the period within which _infections from abroad_ can occur (currently from the second until the end of the eleventh month or in months 7 to 8 or in months 9 to 11).

## EXTENDING THE MODEL

One possible extension would be to close the borders between adjacent `subareas` by always calling the `go-home` procedure when an agent is on one of the patches which are members of the `my-borderline` patch set of the respective `subarea`.

Another possible extension could introduce separate lockdowns per subarea.

## NETLOGO FEATURES

NetLogo 5.3.1 is used instead of NetLogo 6 as the time extension which comes with NetLogo 6 accepts only anonymous procedures, and a former version made ample use of named procedures when scheduling new events. The final version of this model will be converted to NetLogo 6.

## RELATED MODELS

In 2020, a large number of epidemic models were created and published, Any listing would be incomplete these days. The final version of this model will contain such a list.

The normative mode is shaped along the lines of Conte et al. 2014 and Elsenbroich et al. 2016 (particularly the chapters written by Klaus G. Troitzsch in these books).

## CREDITS AND REFERENCES

This model was written by Klaus G. Troitzsch who planneds to use some of its results in an invited talk to be given at the Österreichische Wissenschaftstag in Baden bei Wien on October 20, 2020 (but due to the Covid-19 pandemic, this event was postponed to 2021). A paper will also be published soon. The ODD+D description can be found at https://www.comses.net/users/2399/ together with this model.

Conte, Rosaria, and Giulia Andrighetto and Marco Campennì, eds. (2014) Minding norms.  Oxford and New York: Oxford University Press

Elsenbroich, Corinna, and David Anzola and Nigel Gilbert, eds. (2016) Social Dimensions of Organised Crime. Cham: Springer
@#$#@#$#@
default
false
0
Polygon -7500403 true true 150 0 60 300 150 225 240 300

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

building institution
false
0
Rectangle -7500403 true true 0 60 300 270
Rectangle -16777216 true false 130 196 168 256
Rectangle -16777216 false false 0 255 300 270
Polygon -7500403 true true 0 60 150 15 300 60
Polygon -16777216 false false 0 60 150 15 300 60
Circle -1 true false 135 26 30
Circle -16777216 false false 135 25 30
Rectangle -16777216 false false 0 60 300 75
Rectangle -16777216 false false 218 75 255 90
Rectangle -16777216 false false 218 240 255 255
Rectangle -16777216 false false 224 90 249 240
Rectangle -16777216 false false 45 75 82 90
Rectangle -16777216 false false 45 240 82 255
Rectangle -16777216 false false 51 90 76 240
Rectangle -16777216 false false 90 240 127 255
Rectangle -16777216 false false 90 75 127 90
Rectangle -16777216 false false 96 90 121 240
Rectangle -16777216 false false 179 90 204 240
Rectangle -16777216 false false 173 75 210 90
Rectangle -16777216 false false 173 240 210 255
Rectangle -16777216 false false 269 90 294 240
Rectangle -16777216 false false 263 75 300 90
Rectangle -16777216 false false 263 240 300 255
Rectangle -16777216 false false 0 240 37 255
Rectangle -16777216 false false 6 90 31 240
Rectangle -16777216 false false 0 75 37 90
Line -16777216 false 112 260 184 260
Line -16777216 false 105 265 196 265

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

circle 3
false
15
Circle -1 false true 0 0 300

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cross
false
14
Rectangle -16777216 true true 135 15 165 270
Rectangle -16777216 true true 75 105 195 105
Rectangle -16777216 true true 75 105 225 135
Rectangle -16777216 true true 120 45 180 60

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

hospital
false
0
Rectangle -1 true false 0 0 300 300
Rectangle -2674135 true false 0 105 300 195
Rectangle -2674135 true false 105 0 195 300

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

open square
false
0
Rectangle -7500403 false true 15 15 285 285

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="exp31" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
      <value value="0.65"/>
      <value value="0.7"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
      <value value="0.1"/>
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="55"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="move-speed">
      <value value="0.8"/>
      <value value="1.6"/>
      <value value="1.8"/>
      <value value="2.4"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="25"/>
      <value value="30"/>
      <value value="35"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="exp99" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.7"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="150"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="75"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="move-speed">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.06"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="10000"/>
      <value value="70000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.005"/>
      <value value="0.006"/>
      <value value="0.007"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="baseline" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="0.27"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.45"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="1.33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="3.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="35"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;setup-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-2078345452"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="55"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.75"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="sensitivity" repetitions="40" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;setup-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="55"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="sen04" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
      <value value="&quot;internal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-3"/>
      <value value="0"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;setup-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="35"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
      <value value="0.6"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="sen05" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
      <value value="&quot;internal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-3"/>
      <value value="0"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="35"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
      <value value="0.6"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="sen06" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;external&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-3"/>
      <value value="0"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="35"/>
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
      <value value="0.6"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="mar06" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
      <value value="&quot;internal&quot;"/>
      <value value="&quot;external&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-6"/>
      <value value="0"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="35"/>
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0"/>
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
      <value value="0.6"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="70000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="ulf06" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
      <value value="&quot;internal&quot;"/>
      <value value="&quot;external&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-6"/>
      <value value="0"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
      <value value="&quot;seasonal&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="35"/>
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0"/>
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
      <value value="0.6"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="vac06" repetitions="3" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;setup-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vaccination?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="vac07" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <metric>vaccinated</metric>
    <metric>vaccination-refused</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <steppedValueSet variable="go-seed" first="-1637705880" step="12345" last="-1637656500"/>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vaccination?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="vac08" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <metric>vaccinated</metric>
    <metric>vaccination-refused</metric>
    <metric>vacc-lasts-days</metric>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38149"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vaccination?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vacc-lasts-days">
      <value value="180"/>
      <value value="240"/>
      <value value="300"/>
      <value value="360"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="vac09" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>setup-randomseed</metric>
    <metric>go-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <metric>sdev-distance-to-nearest-neighbour</metric>
    <metric>median-ccs</metric>
    <metric>median-ccns</metric>
    <metric>when-max-infected</metric>
    <metric>n-of-max-infected</metric>
    <metric>when-max-deceased</metric>
    <metric>n-of-max-deceased</metric>
    <metric>never-met</metric>
    <metric>escaped</metric>
    <metric>infected</metric>
    <metric>injected</metric>
    <metric>are-susceptible</metric>
    <metric>are-infected</metric>
    <metric>are-ill</metric>
    <metric>are-recovered</metric>
    <metric>are-immune</metric>
    <metric>are-dead</metric>
    <metric>are-infectious</metric>
    <metric>wear-a-mask</metric>
    <metric>stay-at-home</metric>
    <metric>no-more-infectious-at</metric>
    <metric>mean-daily-moves</metric>
    <metric>ticks</metric>
    <metric>run-length</metric>
    <metric>vaccinated</metric>
    <metric>vaccination-refused</metric>
    <metric>vacc-lasts-days</metric>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown">
      <value value="&quot;off&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.04"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed">
      <value value="&quot;all-constant&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="go-seed">
      <value value="-1637705880"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.65"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;wrt risk&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infection-distribution">
      <value value="&quot;beta 5 2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectiosity-duration">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;holidays&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-subareas">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at%">
      <value value="70"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="move-speed">
      <value value="2.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="setup-seed">
      <value value="-1941496835"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="superspread?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="38000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="travellers">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;sparsest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vaccination?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="vacc-lasts-days">
      <value value="180"/>
      <value value="240"/>
      <value value="300"/>
      <value value="360"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%continuous">
      <value value="20"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
